# -*- coding: utf-8 -*-
"""
KnowledgeLoader 测试
覆盖：多目录扫描、frontmatter 解析、关键词匹配、两级同名项目级优先、
      解析失败跳过、build_index_lines 格式、list_all 返回所有条目。
"""
from pathlib import Path

import pytest

from abyss.knowledge import Knowledge, KnowledgeLoader, create_default_loader


def _write_md(path: Path, name: str, description: str,
              tags: list = None, level: str = "", body: str = "正文",
              category: str = "") -> None:
    """写一个 frontmatter + 正文的 .md 文件到 path。"""
    tags = tags or []
    tags_line = f"[{', '.join(tags)}]" if tags else "[]"
    level_line = f"level: {level}" if level else ""
    category_line = f"category: {category}" if category else ""
    fm = "---\n"
    fm += f"name: {name}\n"
    fm += f"description: {description}\n"
    fm += f"tags: {tags_line}\n"
    if level_line:
        fm += f"{level_line}\n"
    if category_line:
        fm += f"{category_line}\n"
    fm += "---\n"
    fm += body
    path.write_text(fm, encoding="utf-8")


def _write_no_frontmatter(path: Path, body: str = "无 frontmatter 内容") -> None:
    """写一个无 frontmatter 的 .md 文件。"""
    path.write_text(body, encoding="utf-8")


class TestDiscover:
    """discover 多目录扫描测试。"""

    def test_scan_both_dirs(self, tmp_path):
        global_dir = tmp_path / "global"
        project_dir = tmp_path / "project"
        global_dir.mkdir()
        project_dir.mkdir()
        _write_md(global_dir / "a.md", "knowledge-a", "全局知识 A")
        _write_md(project_dir / "b.md", "knowledge-b", "项目知识 B")

        loader = KnowledgeLoader({"global": global_dir, "project": project_dir})
        loader.discover()

        assert loader.get("knowledge-a") is not None
        assert loader.get("knowledge-b") is not None

    def test_project_none_only_global(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(global_dir / "a.md", "knowledge-a", "全局知识 A")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        assert loader.get("knowledge-a") is not None

    def test_nonexistent_dir_skipped(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(global_dir / "a.md", "knowledge-a", "全局知识 A")
        nonexistent = tmp_path / "nonexistent"

        loader = KnowledgeLoader({"global": global_dir, "project": nonexistent})
        loader.discover()  # 不应抛异常

        assert loader.get("knowledge-a") is not None


class TestFrontmatterParse:
    """frontmatter 解析测试。"""

    def test_parse_name_description_tags(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(
            global_dir / "react.md",
            "react-hooks-stale-closure",
            "React Hooks 闭包陷阱的根因与三种修复方案",
            tags=["react", "hooks", "bug"],
            level="global",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        k = loader.get("react-hooks-stale-closure")
        assert k is not None
        assert k.name == "react-hooks-stale-closure"
        assert k.description == "React Hooks 闭包陷阱的根因与三种修复方案"
        assert k.tags == ["react", "hooks", "bug"]
        assert k.content == "正文"
        assert k.level == "global"

    def test_level_overridden_by_directory(self, tmp_path):
        """frontmatter 的 level 字段被目录级别覆盖。"""
        global_dir = tmp_path / "global"
        project_dir = tmp_path / "project"
        global_dir.mkdir()
        project_dir.mkdir()
        # 即便 frontmatter 写 level: project，全局目录下的也应被覆盖为 global
        _write_md(
            global_dir / "a.md",
            "knowledge-a",
            "知识 A",
            level="project",
        )
        _write_md(
            project_dir / "b.md",
            "knowledge-b",
            "知识 B",
            level="global",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": project_dir})
        loader.discover()

        assert loader.get("knowledge-a").level == "global"
        assert loader.get("knowledge-b").level == "project"


class TestMatchByKeyword:
    """关键词匹配测试。"""

    def test_match_react_hooks(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(
            global_dir / "react.md",
            "react-hooks-stale-closure",
            "react hooks 闭包陷阱的根因与修复方案",
            tags=["react", "hooks"],
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        matched = loader.match_by_keyword("react hooks 问题")
        assert len(matched) >= 1
        assert matched[0].name == "react-hooks-stale-closure"

    def test_match_no_hit_returns_empty(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(
            global_dir / "react.md",
            "react-hooks",
            "react hooks 闭包",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        matched = loader.match_by_keyword("今天天气真好")
        assert matched == []

    def test_match_sorted_by_hit_count_desc(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(global_dir / "a.md", "k-a", "react hooks 闭包")
        _write_md(global_dir / "b.md", "k-b", "react")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        matched = loader.match_by_keyword("react hooks 闭包")
        # 命中数多的排前面（k-a 命中 react+hooks+闭包 三个，k-b 命中 react 一个）
        assert matched[0].name == "k-a"


class TestProjectOverridesGlobal:
    """两级同名项目级优先测试。"""

    def test_same_name_project_wins(self, tmp_path):
        global_dir = tmp_path / "global"
        project_dir = tmp_path / "project"
        global_dir.mkdir()
        project_dir.mkdir()
        _write_md(
            global_dir / "react.md",
            "react-hooks",
            "全局级描述",
            body="全局级正文",
        )
        _write_md(
            project_dir / "react.md",
            "react-hooks",
            "项目级描述",
            body="项目级正文",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": project_dir})
        loader.discover()

        k = loader.get("react-hooks")
        assert k is not None
        assert k.level == "project"
        assert k.content == "项目级正文"
        assert k.description == "项目级描述"


class TestParseFailureSkip:
    """解析失败跳过测试。"""

    def test_no_frontmatter_skipped(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(global_dir / "good.md", "good-k", "好的知识")
        _write_no_frontmatter(global_dir / "bad.md", "没有 frontmatter 的文件")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()  # 不应抛异常

        assert loader.get("good-k") is not None
        assert loader.get("bad") is None
        assert loader.get("bad.md") is None

    def test_missing_name_skipped(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        # 缺 name 字段
        global_dir.joinpath("noname.md").write_text(
            "---\ndescription: 缺 name\n---\n正文",
            encoding="utf-8",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        assert loader.list_all() == []

    def test_missing_description_skipped(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        global_dir.joinpath("nodesc.md").write_text(
            "---\nname: no-desc\n---\n正文",
            encoding="utf-8",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        assert loader.list_all() == []


class TestBuildIndexLines:
    """build_index_lines 格式测试。"""

    def test_empty_returns_empty(self, tmp_path):
        loader = KnowledgeLoader({"global": tmp_path, "project": None})
        assert loader.build_index_lines([]) == ""

    def test_global_prefix(self, tmp_path):
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(global_dir / "a.md", "k-a", "全局知识 A")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()
        k = loader.get("k-a")

        line = loader.build_index_lines([k])
        assert line == "[G][exp] k-a: 全局知识 A"

    def test_project_prefix(self, tmp_path):
        project_dir = tmp_path / "project"
        project_dir.mkdir()
        _write_md(project_dir / "b.md", "k-b", "项目知识 B")

        loader = KnowledgeLoader({"global": None, "project": project_dir})
        loader.discover()
        k = loader.get("k-b")

        line = loader.build_index_lines([k])
        assert line == "[P][exp] k-b: 项目知识 B"

    def test_multi_lines_joined_by_newline(self, tmp_path):
        global_dir = tmp_path / "global"
        project_dir = tmp_path / "project"
        global_dir.mkdir()
        project_dir.mkdir()
        _write_md(global_dir / "a.md", "k-a", "全局知识")
        _write_md(project_dir / "b.md", "k-b", "项目知识")

        loader = KnowledgeLoader({"global": global_dir, "project": project_dir})
        loader.discover()

        lines = loader.build_index_lines(loader.list_all())
        assert "[G][exp] k-a: 全局知识" in lines
        assert "[P][exp] k-b: 项目知识" in lines
        assert lines.count("\n") == 1


class TestListAll:
    """list_all 返回所有条目测试。"""

    def test_list_all_count(self, tmp_path):
        global_dir = tmp_path / "global"
        project_dir = tmp_path / "project"
        global_dir.mkdir()
        project_dir.mkdir()
        _write_md(global_dir / "a.md", "k-a", "全局 A")
        _write_md(global_dir / "b.md", "k-b", "全局 B")
        _write_md(project_dir / "c.md", "k-c", "项目 C")

        loader = KnowledgeLoader({"global": global_dir, "project": project_dir})
        loader.discover()

        all_k = loader.list_all()
        assert len(all_k) == 3
        names = {k.name for k in all_k}
        assert names == {"k-a", "k-b", "k-c"}

    def test_list_all_empty_before_discover(self, tmp_path):
        loader = KnowledgeLoader({"global": tmp_path, "project": None})
        assert loader.list_all() == []


class TestCreateDefaultLoader:
    """create_default_loader 工厂测试。"""

    def test_factory_returns_loader_with_correct_dirs(self, tmp_path, monkeypatch):
        # mock Path.home() 到临时目录
        monkeypatch.setattr(Path, "home", lambda: tmp_path)
        loader = create_default_loader(str(tmp_path / "work"))

        assert isinstance(loader, KnowledgeLoader)
        assert loader.knowledge_dirs["global"] == tmp_path / ".abyss" / "knowledge"
        assert loader.knowledge_dirs["project"] == tmp_path / "work" / ".knowledge"


def _write_md_categorized(base_dir: Path, category_dir: str, filename: str,
                          name: str, description: str,
                          tags: list = None, body: str = "正文",
                          category: str = None) -> Path:
    """写一个带 category 的 .md 文件到 base_dir/category_dir/filename。
    返回文件路径。
    """
    cat_dir = base_dir / category_dir
    cat_dir.mkdir(parents=True, exist_ok=True)
    file_path = cat_dir / filename
    tags = tags or []
    tags_line = f"[{', '.join(tags)}]" if tags else "[]"
    fm = "---\n"
    fm += f"name: {name}\n"
    fm += f"description: {description}\n"
    fm += f"tags: {tags_line}\n"
    if category:
        fm += f"category: {category}\n"
    fm += "---\n"
    fm += body
    file_path.write_text(fm, encoding="utf-8")
    return file_path


class TestCategorySubdirScan:
    """分类子目录扫描测试。"""

    def test_scan_preference_subdir(self, tmp_path):
        """preferences/ 子目录下的知识被正确扫描。"""
        global_dir = tmp_path / "global"
        _write_md_categorized(
            global_dir, "preferences", "pref.md",
            "user-likes-type-hints", "用户喜欢用 type hints",
            tags=["python", "typing"], category="preference",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        k = loader.get("user-likes-type-hints")
        assert k is not None
        assert k.category == "preference"
        assert k.level == "global"

    def test_scan_bugfix_subdir(self, tmp_path):
        """bugfixes/ 子目录下的知识被正确扫描。"""
        project_dir = tmp_path / "project"
        _write_md_categorized(
            project_dir, "bugfixes", "bug1.md",
            "spinner-overwrite-bug", "spinner 光标覆盖 bug 的修复方法",
            tags=["spinner", "cursor"], category="bugfix",
        )

        loader = KnowledgeLoader({"global": None, "project": project_dir})
        loader.discover()

        k = loader.get("spinner-overwrite-bug")
        assert k is not None
        assert k.category == "bugfix"
        assert k.level == "project"

    def test_scan_all_four_categories(self, tmp_path):
        """四个分类子目录都能被扫描。"""
        global_dir = tmp_path / "global"
        project_dir = tmp_path / "project"
        _write_md_categorized(global_dir, "preferences", "p1.md", "pref-1", "偏好1", category="preference")
        _write_md_categorized(global_dir, "experiences", "e1.md", "exp-1", "经验1", category="experience")
        _write_md_categorized(global_dir, "tools", "t1.md", "tool-1", "工具1", category="tool")
        _write_md_categorized(project_dir, "bugfixes", "b1.md", "bug-1", "修复1", category="bugfix")

        loader = KnowledgeLoader({"global": global_dir, "project": project_dir})
        loader.discover()

        assert loader.get("pref-1").category == "preference"
        assert loader.get("exp-1").category == "experience"
        assert loader.get("tool-1").category == "tool"
        assert loader.get("bug-1").category == "bugfix"


class TestBackwardCompatRootMd:
    """向后兼容：根目录下的旧 .md 文件默认归类为 experience。"""

    def test_root_md_defaults_to_experience(self, tmp_path):
        """无 category 字段的根目录 .md 被归类为 experience。"""
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        _write_md(global_dir / "old.md", "old-knowledge", "旧知识无分类")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        k = loader.get("old-knowledge")
        assert k is not None
        assert k.category == "experience"

    def test_root_md_with_category_field(self, tmp_path):
        """根目录 .md 但 frontmatter 有 category 字段时，按字段分类。"""
        global_dir = tmp_path / "global"
        global_dir.mkdir()
        global_dir.joinpath("pref.md").write_text(
            "---\nname: root-pref\ndescription: 根目录偏好\ncategory: preference\ntags: []\n---\n正文",
            encoding="utf-8",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        k = loader.get("root-pref")
        assert k is not None
        assert k.category == "preference"


class TestTagsMatch:
    """tags 参与匹配测试。"""

    def test_tags_boost_match_score(self, tmp_path):
        """tags 命中增加匹配权重，使 tags 命中多的排前面。"""
        global_dir = tmp_path / "global"
        _write_md_categorized(
            global_dir, "experiences", "a.md", "k-a",
            "通用编码", tags=["python", "async"],
            category="experience",
        )
        _write_md_categorized(
            global_dir, "experiences", "b.md", "k-b",
            "通用编码", tags=["python", "async", "testing"],
            category="experience",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        # 两个 description 相同，但 k-b 的 tags 多命中 testing
        matched = loader.match_by_keyword("通用编码 python async testing")
        assert len(matched) >= 2
        # k-b 的 tags 命中更多，应排前面
        assert matched[0].name == "k-b"

    def test_tags_only_match_when_desc_misses(self, tmp_path):
        """description 不命中但 tags 命中时也能匹配。"""
        global_dir = tmp_path / "global"
        _write_md_categorized(
            global_dir, "tools", "t1.md", "tool-1",
            "某工具说明", tags=["file_write", "cursor"],
            category="tool",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        # "file_write" 不在 description 中，但在 tags 中
        matched = loader.match_by_keyword("file_write cursor")
        assert len(matched) >= 1
        assert matched[0].name == "tool-1"


class TestCategoryFilter:
    """match_by_keyword 的 category 过滤测试。"""

    def test_filter_by_category(self, tmp_path):
        """指定 category 只返回该类型的知识。"""
        global_dir = tmp_path / "global"
        _write_md_categorized(global_dir, "preferences", "p1.md", "pref-1", "python 偏好", category="preference")
        _write_md_categorized(global_dir, "experiences", "e1.md", "exp-1", "python 经验", category="experience")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        matched = loader.match_by_keyword("python", category="preference")
        assert len(matched) == 1
        assert matched[0].name == "pref-1"
        assert matched[0].category == "preference"

    def test_no_filter_returns_all_categories(self, tmp_path):
        """不指定 category 返回所有类型的知识。"""
        global_dir = tmp_path / "global"
        _write_md_categorized(global_dir, "preferences", "p1.md", "pref-1", "python 偏好", category="preference")
        _write_md_categorized(global_dir, "experiences", "e1.md", "exp-1", "python 经验", category="experience")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()

        matched = loader.match_by_keyword("python")
        assert len(matched) == 2


class TestBuildIndexLinesWithCategory:
    """build_index_lines 分类前缀测试。"""

    def test_index_with_category_prefix(self, tmp_path):
        """索引行包含分类前缀：[G][pref] name: desc"""
        global_dir = tmp_path / "global"
        _write_md_categorized(global_dir, "preferences", "p1.md", "pref-1", "偏好", category="preference")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()
        k = loader.get("pref-1")

        line = loader.build_index_lines([k])
        assert "[G][pref]" in line
        assert "pref-1" in line

    def test_index_bugfix_prefix(self, tmp_path):
        """bugfix 类型的索引行前缀为 [bug]"""
        project_dir = tmp_path / "project"
        _write_md_categorized(project_dir, "bugfixes", "b1.md", "bug-1", "bug修复", category="bugfix")

        loader = KnowledgeLoader({"global": None, "project": project_dir})
        loader.discover()
        k = loader.get("bug-1")

        line = loader.build_index_lines([k])
        assert "[P][bug]" in line

    def test_index_experience_prefix(self, tmp_path):
        """experience 类型的索引行前缀为 [exp]"""
        global_dir = tmp_path / "global"
        _write_md_categorized(global_dir, "experiences", "e1.md", "exp-1", "经验", category="experience")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()
        k = loader.get("exp-1")

        line = loader.build_index_lines([k])
        assert "[G][exp]" in line

    def test_index_tool_prefix(self, tmp_path):
        """tool 类型的索引行前缀为 [tool]"""
        global_dir = tmp_path / "global"
        _write_md_categorized(global_dir, "tools", "t1.md", "tool-1", "工具", category="tool")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()
        k = loader.get("tool-1")

        line = loader.build_index_lines([k])
        assert "[G][tool]" in line


class TestKnowledgeLifecycleFields:
    """Knowledge 生命周期字段测试。"""

    def test_default_lifecycle_fields(self, tmp_path):
        """新建 Knowledge 的生命周期字段有默认值。"""
        global_dir = tmp_path / "global"
        _write_md_categorized(global_dir, "preferences", "p1.md", "pref-1", "偏好", category="preference")

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()
        k = loader.get("pref-1")

        assert k.usage_count == 0
        assert k.last_used_at is None
        assert k.pinned is False

    def test_frontmatter_lifecycle_fields_parsed(self, tmp_path):
        """frontmatter 中的生命周期字段被正确解析。"""
        global_dir = tmp_path / "global"
        global_dir.mkdir(parents=True, exist_ok=True)
        (global_dir / "p1.md").write_text(
            "---\n"
            "name: pref-1\n"
            "description: 偏好\n"
            "category: preference\n"
            "tags: [python]\n"
            "usage_count: 5\n"
            "pinned: true\n"
            "---\n正文",
            encoding="utf-8",
        )

        loader = KnowledgeLoader({"global": global_dir, "project": None})
        loader.discover()
        k = loader.get("pref-1")

        assert k.usage_count == 5
        assert k.pinned is True



# -*- coding: utf-8 -*-
"""TokenCounter 精确 token 计数测试

测试 TokenCounter 类和 SimpleTruncationEngine 的 token 计数集成。
借鉴 zhikuncode-main 的 TokenizerService，使用 tiktoken 精确计算 token 数，
替换原本基于消息条数的估算（长消息单条即可能超限却未触发压缩的问题）。
"""
from unittest.mock import patch, MagicMock

import pytest

from abyss.context_engine import (
    SimpleTruncationEngine,
    TokenCounter,
)


# ---------- TokenCounter 基本计数 ----------


class TestTokenCounterBasic:
    """TokenCounter 基本计数测试"""

    def test_count_text_english(self):
        """英文文本应返回合理 token 数（hello world 约 2-3 token）。"""
        counter = TokenCounter()
        tokens = counter.count_text("hello world")
        assert 2 <= tokens <= 3

    def test_count_text_empty(self):
        """空字符串应返回 0。"""
        counter = TokenCounter()
        assert counter.count_text("") == 0

    def test_count_text_chinese(self):
        """中文文本 token 数应与英文不同（每字约 1-2 token）。"""
        counter = TokenCounter()
        tokens = counter.count_text("你好世界")
        assert tokens > 0
        english_tokens = counter.count_text("hello world")
        # 中文与英文 token 数应不同
        assert tokens != english_tokens


# ---------- count_messages 精确计数 ----------


class TestCountMessages:
    """count_messages 精确计数测试"""

    def test_count_messages_positive(self):
        """消息列表应返回正 token 数。"""
        counter = TokenCounter()
        messages = [
            {"role": "system", "content": "You are helpful."},
            {"role": "user", "content": "Hello"},
        ]
        assert counter.count_messages(messages) > 0

    def test_count_messages_includes_role_overhead(self):
        """count_messages 应比逐条 count_text 之和多（含 role 开销）。"""
        counter = TokenCounter()
        messages = [
            {"role": "system", "content": "You are helpful."},
            {"role": "user", "content": "Hello"},
        ]
        # 仅统计 content 的 token 数
        sum_of_content = sum(
            counter.count_text(m["content"]) for m in messages
        )
        total = counter.count_messages(messages)
        # 含 role 字段开销和每条 4 token 的开销，应严格大于
        assert total > sum_of_content

    def test_count_messages_empty(self):
        """空消息列表应返回 0。"""
        counter = TokenCounter()
        assert counter.count_messages([]) == 0


# ---------- 缓存命中 ----------


class TestTokenCounterCache:
    """TokenCounter encoder 缓存测试"""

    def test_encoder_loaded_only_once(self):
        """同一实例多次调用 count_text 应只加载一次 encoder。"""
        with patch("tiktoken.get_encoding") as mock_get:
            mock_encoder = MagicMock()
            mock_encoder.encode.return_value = [1, 2, 3]
            mock_get.return_value = mock_encoder

            counter = TokenCounter()
            counter.count_text("hello")
            counter.count_text("world")
            counter.count_text("foo bar")

            # get_encoding 只应被调用一次（缓存命中）
            assert mock_get.call_count == 1

    def test_encoder_lazy_load(self):
        """encoder 应懒加载：构造时不立即加载，首次计数时才加载。"""
        with patch("tiktoken.get_encoding") as mock_get:
            mock_encoder = MagicMock()
            mock_encoder.encode.return_value = [1]
            mock_get.return_value = mock_encoder

            counter = TokenCounter()
            # 构造完成，未调用任何计数方法，不应加载 encoder
            assert mock_get.call_count == 0

            counter.count_text("hello")
            assert mock_get.call_count == 1


# ---------- SimpleTruncationEngine 集成 ----------


class TestEngineTokenIntegration:
    """SimpleTruncationEngine 与 TokenCounter 集成测试"""

    def test_update_from_response_uses_api_value(self):
        """update_from_response 应优先使用 API 提供的 prompt_tokens。"""
        engine = SimpleTruncationEngine()
        engine.update_from_response({"prompt_tokens": 1000})
        assert engine.last_prompt_tokens == 1000

    def test_should_compress_local_estimation_long_message(self):
        """无 API usage 时，单条超长消息应触发压缩（即使消息条数少）。"""
        engine = SimpleTruncationEngine(
            context_length=1000,
            threshold_percent=0.75,
        )
        # 单条约 9000 字符的英文消息，token 数远超 750 阈值
        long_content = "The quick brown fox jumps over the lazy dog. " * 200
        engine._messages = [
            {"role": "user", "content": long_content},
        ]
        assert engine.should_compress() is True

    def test_should_compress_local_estimation_short_message(self):
        """无 API usage 时，短消息不应触发压缩。"""
        engine = SimpleTruncationEngine(
            context_length=1000,
            threshold_percent=0.75,
        )
        engine._messages = [
            {"role": "user", "content": "hi"},
        ]
        assert engine.should_compress() is False

    def test_estimate_tokens_returns_positive(self):
        """estimate_tokens 应返回正数。"""
        engine = SimpleTruncationEngine()
        messages = [{"role": "user", "content": "hello world"}]
        assert engine.estimate_tokens(messages) > 0

    def test_estimate_tokens_uses_token_counter(self):
        """estimate_tokens 应与 TokenCounter.count_messages 结果一致。"""
        engine = SimpleTruncationEngine()
        messages = [
            {"role": "system", "content": "You are helpful."},
            {"role": "user", "content": "Hello"},
        ]
        assert engine.estimate_tokens(messages) == engine._token_counter.count_messages(messages)


# ---------- 阈值判断 ----------


class TestThresholdJudgment:
    """阈值判断测试"""

    def test_threshold_tokens_calculated(self):
        """threshold_tokens 应为 context_length * threshold_percent。"""
        engine = SimpleTruncationEngine(
            context_length=1000,
            threshold_percent=0.75,
        )
        assert engine.threshold_tokens == 750

    def test_compress_triggered_at_threshold(self):
        """消息 token 数 >= 阈值（750）时应触发压缩。"""
        engine = SimpleTruncationEngine(
            context_length=1000,
            threshold_percent=0.75,
        )
        # 构造 >= 750 token 的消息（约 4500 字符英文，~1200 token）
        long_content = "The quick brown fox jumps over the lazy dog. " * 100
        engine._messages = [
            {"role": "user", "content": long_content},
        ]
        assert engine.should_compress() is True

    def test_compress_not_triggered_below_threshold(self):
        """消息 token 数 < 阈值时不应触发压缩。"""
        engine = SimpleTruncationEngine(
            context_length=1000,
            threshold_percent=0.75,
        )
        engine._messages = [
            {"role": "user", "content": "short message"},
        ]
        assert engine.should_compress() is False



# -*- coding: utf-8 -*-
"""
/redo 命令测试：恢复被 /undo 撤销的修改。

覆盖：
1. handle_redo 基本流程：undo 后 redo 恢复文件内容
2. redo 找不到 rewound 事务时返回提示
3. redo 未 undo 的事务应失败
4. redo 后再次 redo 同一事务应失败（rewound=False）
5. redo 新建文件（old_content is None）：从 .bak 恢复文件
6. redo 多文件事务
7. redo 后文件内容等于 AI 编辑后的版本
8. undo→redo→undo 链式操作
9. handle_redo 无 history service 时返回错误
10. handle_redo 无 session_id 时 fallback 到 "default"
"""
import pytest

from abyss.commands.undo import handle_undo, handle_redo, init_history_service
from abyss.history import FileHistoryService


class _FakeSession:
    """测试用的最小 Session。"""

    def __init__(self, session_id: str = "test-session"):
        self.session_id = session_id
        self.messages = []


@pytest.fixture(autouse=True)
def reset_history_service():
    """每个测试前后重置全局单例。"""
    init_history_service(FileHistoryService(storage_dir="__test_redo_reset__"))
    yield
    init_history_service(FileHistoryService(storage_dir="__test_redo_reset__"))


# ---------- handle_redo 基本流程 ----------

def test_redo_restores_file_content(tmp_path):
    """undo 后 redo 应恢复文件到 AI 编辑后的版本。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    target = tmp_path / "file.py"
    target.write_text("original", encoding="utf-8")
    # 模拟 AI 编辑：原内容 "original" → 新内容 "edited"
    tid = svc.begin_transaction("s1", "m1")
    svc.record_file_change(tid, str(target), "original")
    svc.commit_transaction(tid)
    target.write_text("edited", encoding="utf-8")

    session = _FakeSession(session_id="s1")
    # undo：文件回退到 "original"
    r_undo = handle_undo(session)
    assert "[OK]" in r_undo
    assert target.read_text(encoding="utf-8") == "original"

    # redo：文件恢复到 "edited"
    r_redo = handle_redo(session)
    assert "[OK]" in r_redo
    assert "影响文件 1 个" in r_redo
    assert target.read_text(encoding="utf-8") == "edited"


def test_redo_no_rewound_transaction_returns_info(tmp_path):
    """无被撤销的事务时返回 [INFO] 提示。"""
    svc = FileHistoryService(storage_dir=str(tmp_path))
    init_history_service(svc)

    session = _FakeSession(session_id="s1")
    result = handle_redo(session)

    assert "[INFO]" in result
    assert "无可恢复" in result or "/undo" in result


def test_redo_not_rewound_transaction_fails(tmp_path):
    """直接 redo 一个未撤销的事务应失败。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    target = tmp_path / "file.py"
    target.write_text("original", encoding="utf-8")
    tid = svc.begin_transaction("s1", "m1")
    svc.record_file_change(tid, str(target), "original")
    svc.commit_transaction(tid)
    target.write_text("edited", encoding="utf-8")

    # 直接 redo（未先 undo）
    result = svc.redo(tid)

    assert result.success is False
    assert "not rewound" in result.error


def test_redo_same_transaction_twice_fails(tmp_path):
    """redo 后再次 redo 同一事务应失败（rewound 已翻回 False）。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    target = tmp_path / "file.py"
    target.write_text("original", encoding="utf-8")
    tid = svc.begin_transaction("s1", "m1")
    svc.record_file_change(tid, str(target), "original")
    svc.commit_transaction(tid)
    target.write_text("edited", encoding="utf-8")

    # undo → redo → 再 redo（应失败）
    svc.rewind(tid)
    r1 = svc.redo(tid)
    assert r1.success is True

    r2 = svc.redo(tid)
    assert r2.success is False
    assert "not rewound" in r2.error


# ---------- 新建文件 redo ----------

def test_redo_new_file_restores_from_bak(tmp_path):
    """新建文件（old_content is None）：undo 删除后 redo 应从 .bak 恢复文件。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    new_file = tmp_path / "new.py"
    # 模拟 AI 新建文件：old_content=None
    tid = svc.begin_transaction("s1", "m1")
    svc.record_file_change(tid, str(new_file), None)  # None 表示新建
    svc.commit_transaction(tid)
    new_file.write_text("created by AI", encoding="utf-8")

    session = _FakeSession(session_id="s1")
    # undo：文件被删除
    r_undo = handle_undo(session)
    assert "[OK]" in r_undo
    assert not new_file.exists()

    # redo：文件从 .bak 恢复
    r_redo = handle_redo(session)
    assert "[OK]" in r_redo
    assert new_file.exists()
    assert new_file.read_text(encoding="utf-8") == "created by AI"


# ---------- 多文件 redo ----------

def test_redo_multiple_files(tmp_path):
    """redo 多文件事务：所有文件都应恢复。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    a = tmp_path / "a.py"
    b = tmp_path / "b.py"
    a.write_text("a-orig", encoding="utf-8")
    b.write_text("b-orig", encoding="utf-8")

    tid = svc.begin_transaction("s1", "m1")
    svc.record_file_change(tid, str(a), "a-orig")
    svc.record_file_change(tid, str(b), "b-orig")
    svc.commit_transaction(tid)
    a.write_text("a-edited", encoding="utf-8")
    b.write_text("b-edited", encoding="utf-8")

    session = _FakeSession(session_id="s1")
    handle_undo(session)  # 回退到 orig

    r_redo = handle_redo(session)
    assert "[OK]" in r_redo
    assert "影响文件 2 个" in r_redo
    assert a.read_text(encoding="utf-8") == "a-edited"
    assert b.read_text(encoding="utf-8") == "b-edited"


# ---------- 链式操作 ----------

def test_undo_redo_undo_chain(tmp_path):
    """undo → redo → undo 链式操作，每次状态正确。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    target = tmp_path / "file.py"
    target.write_text("original", encoding="utf-8")
    tid = svc.begin_transaction("s1", "m1")
    svc.record_file_change(tid, str(target), "original")
    svc.commit_transaction(tid)
    target.write_text("edited", encoding="utf-8")

    session = _FakeSession(session_id="s1")

    # 第一次 undo：edited → original
    handle_undo(session)
    assert target.read_text(encoding="utf-8") == "original"

    # redo：original → edited
    handle_redo(session)
    assert target.read_text(encoding="utf-8") == "edited"

    # 第二次 undo：edited → original
    handle_undo(session)
    assert target.read_text(encoding="utf-8") == "original"


# ---------- 边界情况 ----------

def test_redo_no_history_service_returns_error():
    """无 history service 时返回错误。"""
    import abyss.commands.undo as undo_mod
    old = undo_mod._HISTORY_SERVICE
    undo_mod._HISTORY_SERVICE = None
    try:
        session = _FakeSession()
        result = handle_redo(session)
        assert "[ERR]" in result
    finally:
        undo_mod._HISTORY_SERVICE = old


def test_redo_no_session_id_falls_back_to_default(tmp_path):
    """session 无 session_id 时回退到 "default"。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    target = tmp_path / "file.py"
    target.write_text("orig", encoding="utf-8")
    # 用 "default" session_id 创建事务（模拟 main.py 行为）
    tid = svc.begin_transaction("default", "m1")
    svc.record_file_change(tid, str(target), "orig")
    svc.commit_transaction(tid)
    target.write_text("edited", encoding="utf-8")

    # 先 undo（用无 session_id 的 session）
    class _SessionNoId:
        messages = []
    r_undo = handle_undo(_SessionNoId())
    assert "[OK]" in r_undo

    # 再 redo
    r_redo = handle_redo(_SessionNoId())
    assert "[OK]" in r_redo
    assert target.read_text(encoding="utf-8") == "edited"


def test_redo_scoped_to_session(tmp_path):
    """redo 按 session 隔离：session A 的 undo 不影响 session B 的 redo。"""
    svc = FileHistoryService(storage_dir=str(tmp_path / "hist"))
    init_history_service(svc)

    # session A 的事务
    a_target = tmp_path / "a.py"
    a_target.write_text("a-orig", encoding="utf-8")
    tid_a = svc.begin_transaction("sess-a", "m1")
    svc.record_file_change(tid_a, str(a_target), "a-orig")
    svc.commit_transaction(tid_a)
    a_target.write_text("a-edited", encoding="utf-8")

    # session B 的事务
    b_target = tmp_path / "b.py"
    b_target.write_text("b-orig", encoding="utf-8")
    tid_b = svc.begin_transaction("sess-b", "m1")
    svc.record_file_change(tid_b, str(b_target), "b-orig")
    svc.commit_transaction(tid_b)
    b_target.write_text("b-edited", encoding="utf-8")

    # session A undo
    session_a = _FakeSession(session_id="sess-a")
    handle_undo(session_a)
    assert a_target.read_text(encoding="utf-8") == "a-orig"

    # session B redo（应失败：session B 没有 rewound 事务）
    session_b = _FakeSession(session_id="sess-b")
    r_b = handle_redo(session_b)
    assert "[INFO]" in r_b  # 无可恢复的撤销

    # b.py 不应被改动
    assert b_target.read_text(encoding="utf-8") == "b-edited"



# -*- coding: utf-8 -*-
"""
回归测试：/session 菜单选择会话后名字解析错误。

Bug 复现：main.py 用 `selected[3:]` 切片去前缀 "📂 "，
但 Python 中 emoji "📂" 只占 1 个字符 + 1 空格 = 2 字符，
selected[3:] 会少取一个字符（如 "你好git..." → "好git..."），
导致 SESSION_DIR / f"{name}.json" 找不到文件，from_file 返回空 session，
用户看到 "0 条消息"。
"""
import pytest


def test_session_name_parsing_from_menu_selection():
    """从菜单选中项解析会话名应正确，不应丢失字符。"""
    # 复现 main.py:967 的 bug
    selected = "📂 你好git提交当前修改_0718-2120"

    # 修复后应使用 split(" ", 1)[1] 或 selected[2:]
    # 旧 bug: selected[3:] → "好git提交当前修改_0718-2120" (少了 "你")
    # 新: 应得到 "你好git提交当前修改_0718-2120"
    from abyss.main import _parse_session_name_from_selection

    name = _parse_session_name_from_selection(selected)
    assert name == "你好git提交当前修改_0718-2120"
    assert "你" in name  # 旧 bug 会丢失首字


def test_session_name_with_ascii():
    """纯 ASCII 会话名也应正确解析。"""
    selected = "📂 session_0706-0029"

    from abyss.main import _parse_session_name_from_selection
    name = _parse_session_name_from_selection(selected)

    assert name == "session_0706-0029"


def test_session_name_with_emoji_in_name():
    """会话名本身含 emoji 时也应正确（auto_name 会过滤，但防御）。"""
    selected = "📂 测试🎯会话"

    from abyss.main import _parse_session_name_from_selection
    name = _parse_session_name_from_selection(selected)

    assert name == "测试🎯会话"



# -*- coding: utf-8 -*-
"""
方案 B 测试：todo_write 状态管理与 main.py 注入逻辑。

覆盖：
1. todo_write.execute 后模块级状态正确更新（replace / merge 两种模式）
2. get_active_todos() 返回浅拷贝，外部修改不影响内部状态
3. clear_todos() 清空状态
4. _inject_todos_into_messages：无 todo 时不注入（零浪费）
5. _inject_todos_into_messages：有未完成项时注入到 system 末尾
6. _inject_todos_into_messages：全部完成时清空状态且不注入
7. _inject_todos_into_messages：不污染原 messages 列表
8. _inject_todos_into_messages：messages 无 system 时首位补 system
"""
import pytest

from abyss.tools.todo_write import (
    TodoWriteTool,
    get_active_todos,
    clear_todos,
)


@pytest.fixture(autouse=True)
def _reset_todo_state():
    """每用例前清空模块级 todo 状态，避免用例间相互污染。"""
    clear_todos()
    yield
    clear_todos()


def _todo(tid, content, status="pending", priority="medium"):
    return {"id": tid, "content": content, "status": status, "priority": priority}


# ---------- 状态管理 ----------

def test_execute_replace_updates_state():
    """replace 模式：execute 后模块级状态应为传入的 todos。"""
    tool = TodoWriteTool()
    todos = [_todo("1", "任务A", "pending"), _todo("2", "任务B", "in_progress")]

    tool.execute(todos=todos, merge=False)

    active = get_active_todos()
    assert len(active) == 2
    assert {t["id"] for t in active} == {"1", "2"}


def test_execute_merge_updates_by_id():
    """merge 模式：按 id 合并更新，未在传入列表中的项保留。"""
    tool = TodoWriteTool()
    # 初始化：两条 pending
    tool.execute(todos=[_todo("1", "任务A", "pending"), _todo("2", "任务B", "pending")], merge=False)
    # merge 更新：把 id=1 改为 completed
    tool.execute(todos=[_todo("1", "任务A", "completed")], merge=True)

    active = get_active_todos()
    assert len(active) == 2  # id=2 仍保留
    by_id = {t["id"]: t for t in active}
    assert by_id["1"]["status"] == "completed"
    assert by_id["2"]["status"] == "pending"


def test_get_active_todos_returns_copy():
    """get_active_todos() 返回浅拷贝，外部修改不应影响内部状态。"""
    tool = TodoWriteTool()
    tool.execute(todos=[_todo("1", "任务A", "pending")], merge=False)

    snapshot = get_active_todos()
    snapshot[0]["status"] = "completed"  # 外部修改

    # 内部状态不应被影响
    assert get_active_todos()[0]["status"] == "pending"


def test_clear_todos_empties_state():
    """clear_todos() 后 get_active_todos() 应返回空列表。"""
    tool = TodoWriteTool()
    tool.execute(todos=[_todo("1", "任务A", "pending")], merge=False)
    assert len(get_active_todos()) == 1

    clear_todos()

    assert get_active_todos() == []


# ---------- 注入逻辑 ----------

def test_inject_no_todos_returns_unchanged():
    """无 todo 状态时，_inject_todos_into_messages 应原样返回（不注入）。"""
    from abyss.main import _inject_todos_into_messages

    messages = [{"role": "system", "content": "原始 system"}, {"role": "user", "content": "hi"}]

    result = _inject_todos_into_messages(messages)

    assert result == messages
    assert result[0]["content"] == "原始 system"


def test_inject_with_incomplete_todos_appends_to_system():
    """有未完成 todo 时，应追加到 system 消息末尾。"""
    from abyss.main import _inject_todos_into_messages
    tool = TodoWriteTool()
    tool.execute(todos=[
        _todo("1", "任务A", "pending"),
        _todo("2", "任务B", "in_progress"),
    ], merge=False)

    messages = [{"role": "system", "content": "原始 system"}, {"role": "user", "content": "hi"}]
    result = _inject_todos_into_messages(messages)

    assert "原始 system" in result[0]["content"]
    assert "任务A" in result[0]["content"]
    assert "任务B" in result[0]["content"]
    assert "当前任务进度" in result[0]["content"]


def test_inject_all_completed_clears_state_and_skips():
    """全部完成时：清空状态且不注入（避免无意义信息）。"""
    from abyss.main import _inject_todos_into_messages
    tool = TodoWriteTool()
    tool.execute(todos=[_todo("1", "任务A", "completed")], merge=False)

    messages = [{"role": "system", "content": "原始 system"}]
    result = _inject_todos_into_messages(messages)

    # 不注入
    assert result == messages
    # 状态被清空
    assert get_active_todos() == []


def test_inject_does_not_mutate_original_messages():
    """注入应在副本上操作，不污染原 messages 列表。"""
    from abyss.main import _inject_todos_into_messages
    tool = TodoWriteTool()
    tool.execute(todos=[_todo("1", "任务A", "pending")], merge=False)

    original = [{"role": "system", "content": "原始 system"}, {"role": "user", "content": "hi"}]
    _inject_todos_into_messages(original)

    # 原 messages 不应被修改
    assert original[0]["content"] == "原始 system"
    assert len(original) == 2


def test_inject_no_system_message_prepends_one():
    """messages 无 system 时，应在首位补一条 system。"""
    from abyss.main import _inject_todos_into_messages
    tool = TodoWriteTool()
    tool.execute(todos=[_todo("1", "任务A", "pending")], merge=False)

    messages = [{"role": "user", "content": "hi"}]
    result = _inject_todos_into_messages(messages)

    assert result[0]["role"] == "system"
    assert "任务A" in result[0]["content"]
    assert result[1] == messages[0]


def test_inject_orders_in_progress_first():
    """注入时应按状态排序：in_progress > pending > completed。"""
    from abyss.main import _inject_todos_into_messages
    tool = TodoWriteTool()
    tool.execute(todos=[
        _todo("3", "已完成", "completed"),
        _todo("1", "进行中", "in_progress"),
        _todo("2", "待办", "pending"),
    ], merge=False)

    messages = [{"role": "system", "content": "sys"}]
    result = _inject_todos_into_messages(messages)
    injected_text = result[0]["content"]

    # in_progress 应出现在 pending 和 completed 之前
    idx_in_progress = injected_text.index("进行中")
    idx_pending = injected_text.index("待办")
    idx_completed = injected_text.index("已完成")
    assert idx_in_progress < idx_pending < idx_completed



# -*- coding: utf-8 -*-
"""CommitAttributionService Git 提交归因测试（Phase 7 Tier 1 BF）。

借鉴 zhikuncode-main CommitAttributionService.java：
- git blame 解析（行级作者 + 时间）
- git log 解析（最近 N 条提交）
- git shortlog 解析（按作者统计提交数）
- 提交摘要（commit + author + date + message 首行）
"""
import os
import subprocess
from pathlib import Path

import pytest

from abyss.vcs.commit_attribution import (
    CommitAttributionService,
    BlameLine,
    CommitInfo,
    AuthorStats,
    get_default_service,
)


@pytest.fixture
def git_repo(tmp_path: Path) -> str:
    """创建临时 git 仓库并写入若干提交。"""
    repo = str(tmp_path / "repo")
    os.makedirs(repo, exist_ok=True)
    env = os.environ.copy()
    env["GIT_AUTHOR_NAME"] = "Tester1"
    env["GIT_AUTHOR_EMAIL"] = "tester1@example.com"
    env["GIT_COMMITTER_NAME"] = "Tester1"
    env["GIT_COMMITTER_EMAIL"] = "tester1@example.com"
    env["GIT_AUTHOR_DATE"] = "2025-01-01T10:00:00"
    env["GIT_COMMITTER_DATE"] = "2025-01-01T10:00:00"

    def _git(*args):
        return subprocess.run(
            ["git", *args], cwd=repo, capture_output=True, text=True,
            env=env, timeout=10,
        )

    _git("init", "-q")
    _git("config", "user.name", "Tester1")
    _git("config", "user.email", "tester1@example.com")

    # 第一次提交
    (Path(repo) / "file.py").write_text("line1\nline2\nline3\n", encoding="utf-8")
    _git("add", "file.py")
    _git("commit", "-q", "-m", "first commit")

    # 第二次提交（不同作者）
    env["GIT_AUTHOR_NAME"] = "Tester2"
    env["GIT_AUTHOR_EMAIL"] = "tester2@example.com"
    env["GIT_COMMITTER_NAME"] = "Tester2"
    env["GIT_COMMITTER_EMAIL"] = "tester2@example.com"
    env["GIT_AUTHOR_DATE"] = "2025-01-02T11:00:00"
    env["GIT_COMMITTER_DATE"] = "2025-01-02T11:00:00"
    (Path(repo) / "file.py").write_text("line1\nline2 modified\nline3\n", encoding="utf-8")
    _git("add", "file.py")
    _git("commit", "-q", "-m", "second commit")

    return repo


@pytest.fixture
def service(git_repo: str) -> CommitAttributionService:
    return CommitAttributionService(working_dir=git_repo)


class TestBlameParse:
    """git blame 解析测试。"""

    def test_blame_returns_lines(self, service, git_repo):
        """blame 应返回每行的作者信息。"""
        result = service.blame("file.py")
        assert len(result) == 3
        for line in result:
            assert isinstance(line, BlameLine)
            assert line.line_number >= 1
            assert line.author in ("Tester1", "Tester2")
            assert line.commit_hash  # 40 字符 SHA

    def test_blame_specific_lines(self, service, git_repo):
        """指定行范围应只返回该范围。"""
        result = service.blame("file.py", start=1, end=2)
        assert len(result) == 2
        assert result[0].line_number == 1
        assert result[1].line_number == 2

    def test_blame_modified_line_author(self, service, git_repo):
        """第二行被 Tester2 修改，作者应是 Tester2。"""
        result = service.blame("file.py")
        # line2 被第二次提交修改
        line2 = next(l for l in result if l.line_number == 2)
        assert line2.author == "Tester2"

    def test_blame_nonexistent_file_returns_empty(self, service, git_repo):
        """blame 不存在的文件应返回空列表，不抛异常。"""
        result = service.blame("nonexistent.py")
        assert result == []


class TestLogParse:
    """git log 解析测试。"""

    def test_log_returns_recent_commits(self, service, git_repo):
        """log 默认返回最近 10 条提交。"""
        commits = service.log(limit=10)
        assert len(commits) == 2  # 仓库里只有 2 个提交
        for c in commits:
            assert isinstance(c, CommitInfo)
            assert c.hash
            assert c.author
            assert c.date
            assert c.message

    def test_log_returns_in_reverse_chronological(self, service, git_repo):
        """log 应按时间倒序（最近在前）。"""
        commits = service.log(limit=10)
        assert "second" in commits[0].message
        assert "first" in commits[1].message

    def test_log_limit_1(self, service, git_repo):
        """limit=1 应只返回最近一条。"""
        commits = service.log(limit=1)
        assert len(commits) == 1
        assert "second" in commits[0].message


class TestAuthorStats:
    """按作者统计提交数测试。"""

    def test_shortlog_groups_by_author(self, service, git_repo):
        """按作者分组统计提交数。"""
        stats = service.author_stats()
        assert len(stats) == 2  # Tester1 + Tester2
        authors = {s.author for s in stats}
        assert "Tester1" in authors
        assert "Tester2" in authors
        for s in stats:
            assert s.commit_count == 1


class TestCommitSummary:
    """提交摘要测试。"""

    def test_commit_summary_returns_full_info(self, service, git_repo):
        """获取指定 commit 摘要。"""
        commits = service.log(limit=1)
        sha = commits[0].hash
        summary = service.commit_summary(sha)
        assert summary is not None
        assert summary.hash == sha
        assert summary.author == "Tester2"
        assert "second" in summary.message

    def test_commit_summary_nonexistent_returns_none(self, service, git_repo):
        """不存在的 commit 返回 None。"""
        assert service.commit_summary("nonexistent_sha") is None


class TestNonGitRepo:
    """非 git 仓库场景。"""

    def test_blame_in_non_git_returns_empty(self, tmp_path):
        svc = CommitAttributionService(working_dir=str(tmp_path))
        assert svc.blame("any.py") == []

    def test_log_in_non_git_returns_empty(self, tmp_path):
        svc = CommitAttributionService(working_dir=str(tmp_path))
        assert svc.log() == []

    def test_author_stats_in_non_git_returns_empty(self, tmp_path):
        svc = CommitAttributionService(working_dir=str(tmp_path))
        assert svc.author_stats() == []

    def test_is_git_repo_detects_git_directory(self, git_repo, tmp_path):
        svc = CommitAttributionService(working_dir=git_repo)
        assert svc.is_git_repo() is True
        svc2 = CommitAttributionService(working_dir=str(tmp_path))
        assert svc2.is_git_repo() is False


class TestDefaultService:
    """模块级单例测试。"""

    def test_get_default_returns_same_instance(self):
        s1 = get_default_service()
        s2 = get_default_service()
        assert s1 is s2



# -*- coding: utf-8 -*-
"""ToolRegistry _tool_source 追溯到测试"""
import pytest
from abyss.tools.registry import ToolRegistry
from abyss.tools.base import Tool


class FakeTool(Tool):
    def __init__(self, name):
        self._name = name
    @property
    def name(self) -> str:
        return self._name
    @property
    def description(self) -> str:
        return "fake"
    @property
    def parameters(self) -> dict:
        return {"type": "object", "properties": {}}
    def execute(self, **kw) -> dict:
        return {"ok": True}


class TestToolSource:
    """工具来源追溯测试"""

    def test_register_with_source(self):
        reg = ToolRegistry()
        reg.register(FakeTool("t1"), source="builtin")
        assert reg.get_source("t1") == "builtin"

    def test_register_without_source(self):
        reg = ToolRegistry()
        reg.register(FakeTool("t1"))
        assert reg.get_source("t1") == "unknown"

    def test_list_by_source(self):
        reg = ToolRegistry()
        reg.register(FakeTool("t1"), source="builtin")
        reg.register(FakeTool("t2"), source="mcp__github")
        reg.register(FakeTool("t3"), source="builtin")

        builtin = reg.list_by_source("builtin")
        assert set(builtin) == {"t1", "t3"}

        mcp = reg.list_by_source("mcp__github")
        assert mcp == ["t2"]

    def test_unregister_removes_source(self):
        reg = ToolRegistry()
        reg.register(FakeTool("t1"), source="builtin")
        reg.unregister("t1")
        assert reg.get_source("t1") is None

    def test_unregister_by_source(self):
        reg = ToolRegistry()
        reg.register(FakeTool("t1"), source="mcp__s1")
        reg.register(FakeTool("t2"), source="mcp__s1")
        reg.register(FakeTool("t3"), source="mcp__s2")

        removed = reg.unregister_by_source("mcp__s1")
        assert removed == 2
        assert "t1" not in reg
        assert "t2" not in reg
        assert "t3" in reg  # 不同源不受影响

    def test_get_source_unknown(self):
        reg = ToolRegistry()
        assert reg.get_source("nonexistent") is None



# -*- coding: utf-8 -*-
"""ToolCallTracker 单元测试

覆盖：
- ToolCallRecord.create() args_hash 计算
- record() 基本记录
- is_window_all_failed() 滑动窗口
- is_same_error_repeated() 同错误重复
- consecutive_errors() 计数
- recent() / total_count() / reset() / get_status()
- recovery_relevant 标志位
- FIFO 淘汰
- 线程安全
- 默认单例
"""
import json
import threading
import time

import pytest

from abyss.tools.tracking import (
    ToolCallRecord,
    ToolCallTracker,
    get_default_tracker,
    reset_default_tracker,
)


# ---------------- 工具函数 ----------------

def make_args(**kwargs):
    """构造工具参数 dict。"""
    return dict(kwargs)


# ---------------- ToolCallRecord.create() ----------------

class TestToolCallRecordCreate:
    def test_args_hash_stable_for_same_args(self):
        """相同参数（顺序不同）应生成相同 hash。"""
        r1 = ToolCallRecord.create("file_edit", {"path": "a.py", "old": "x"}, False)
        r2 = ToolCallRecord.create("file_edit", {"old": "x", "path": "a.py"}, False)
        assert r1.args_hash == r2.args_hash

    def test_args_hash_differs_for_different_args(self):
        """不同参数应生成不同 hash。"""
        r1 = ToolCallRecord.create("file_edit", {"path": "a.py"}, False)
        r2 = ToolCallRecord.create("file_edit", {"path": "b.py"}, False)
        assert r1.args_hash != r2.args_hash

    def test_args_hash_for_unhashable_args(self):
        """不可序列化的参数应回退到 'unhashable'。"""
        class Unserializable:
            pass

        # 直接传入不可序列化对象
        r = ToolCallRecord.create(
            "shell_exec", {"obj": Unserializable()}, False
        )
        # 因 json.dumps 默认不可序列化会抛 TypeError
        assert r.args_hash == "unhashable"

    def test_create_with_all_fields(self):
        """完整字段创建应正确赋值。"""
        r = ToolCallRecord.create(
            "file_edit", {"path": "a.py"}, False,
            error="file not found", error_type="FileNotFoundError",
            recovery_relevant=False,
        )
        assert r.tool_name == "file_edit"
        assert r.success is False
        assert r.error_message == "file not found"
        assert r.error_type == "FileNotFoundError"
        assert r.recovery_relevant is False
        assert r.timestamp > 0


# ---------------- 基础 record() ----------------

class TestRecord:
    def test_record_success_increments_total(self):
        tracker = ToolCallTracker()
        tracker.record("file_read", {"path": "a.py"}, True)
        assert tracker.total_count() == 1

    def test_record_failure_increments_total(self):
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="not found")
        assert tracker.total_count() == 1

    def test_record_success_resets_consecutive_errors(self):
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err1")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err2")
        assert tracker.consecutive_errors() == 2
        tracker.record("file_read", {"path": "a.py"}, True)
        assert tracker.consecutive_errors() == 0


# ---------------- is_window_all_failed() ----------------

class TestWindowAllFailed:
    def test_empty_tracker_returns_false(self):
        tracker = ToolCallTracker()
        assert tracker.is_window_all_failed(3) is False

    def test_insufficient_history_returns_false(self):
        """历史不足 window_size 时返回 False。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        assert tracker.is_window_all_failed(3) is False

    def test_all_failed_window_true(self):
        """最近 N 次全失败返回 True。"""
        tracker = ToolCallTracker()
        for i in range(3):
            tracker.record("file_edit", {"path": f"a{i}.py"}, False, error=f"err{i}")
        assert tracker.is_window_all_failed(3) is True

    def test_mixed_success_failure_returns_false(self):
        """最近 N 次中有成功返回 False。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_read", {"path": "a.py"}, True)
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        assert tracker.is_window_all_failed(3) is False

    def test_recovery_relevant_false_excluded(self):
        """recovery_relevant=False 的记录不计入全失败判定。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="blocked", recovery_relevant=False)
        tracker.record("file_edit", {"path": "a.py"}, False, error="blocked", recovery_relevant=False)
        tracker.record("file_edit", {"path": "a.py"}, False, error="blocked", recovery_relevant=False)
        # 全是预期终态，不算"需要恢复"
        assert tracker.is_window_all_failed(3) is False

    def test_window_size_5(self):
        """window_size=5 时的判定。"""
        tracker = ToolCallTracker()
        for i in range(4):
            tracker.record("file_edit", {"path": f"a{i}.py"}, False, error="err")
        # 4 次不足 5
        assert tracker.is_window_all_failed(5) is False
        tracker.record("file_edit", {"path": "a4.py"}, False, error="err")
        assert tracker.is_window_all_failed(5) is True


# ---------------- is_same_error_repeated() ----------------

class TestSameErrorRepeated:
    def test_empty_tracker_returns_false(self):
        tracker = ToolCallTracker()
        assert tracker.is_same_error_repeated(3) is False

    def test_threshold_zero_returns_false(self):
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        assert tracker.is_same_error_repeated(0) is False

    def test_latest_success_returns_false(self):
        """最新一次成功，不算重复失败。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_read", {"path": "a.py"}, True)
        assert tracker.is_same_error_repeated(3) is False

    def test_same_error_3_times_true(self):
        tracker = ToolCallTracker()
        for _ in range(3):
            tracker.record("file_edit", {"path": "a.py"}, False, error="same error")
        assert tracker.is_same_error_repeated(3) is True

    def test_different_errors_returns_false(self):
        """不同 error_message 连续出现不算重复。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err1")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err2")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err3")
        assert tracker.is_same_error_repeated(3) is False

    def test_success_breaks_continuity(self):
        """成功记录打断连续性。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_read", {"path": "a.py"}, True)  # 打断
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        # 最新一次 err 但前面是 success，连续数=1 < 3
        assert tracker.is_same_error_repeated(3) is False

    def test_threshold_2(self):
        """threshold=2 时的判定。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        assert tracker.is_same_error_repeated(2) is True
        assert tracker.is_same_error_repeated(3) is False


# ---------------- consecutive_errors() ----------------

class TestConsecutiveErrors:
    def test_initial_zero(self):
        tracker = ToolCallTracker()
        assert tracker.consecutive_errors() == 0

    def test_increment_on_failure(self):
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        assert tracker.consecutive_errors() == 1
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        assert tracker.consecutive_errors() == 2

    def test_reset_on_success(self):
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_read", {"path": "a.py"}, True)
        assert tracker.consecutive_errors() == 0

    def test_recovery_relevant_false_still_increments(self):
        """recovery_relevant=False 的失败仍参与计数（不影响 consecutive_errors）。"""
        tracker = ToolCallTracker()
        tracker.record(
            "file_edit", {"path": "a.py"}, False, error="blocked",
            recovery_relevant=False,
        )
        assert tracker.consecutive_errors() == 1


# ---------------- recent() / total_count() / reset() ----------------

class TestRecentAndReset:
    def test_recent_returns_copy(self):
        """recent() 返回副本，修改不影响内部状态。"""
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        recent = tracker.recent(5)
        assert len(recent) == 1
        recent.clear()
        # 内部未受影响
        assert tracker.total_count() == 1

    def test_recent_n_returns_last_n(self):
        tracker = ToolCallTracker()
        for i in range(10):
            tracker.record("file_edit", {"path": f"a{i}.py"}, True)
        recent = tracker.recent(3)
        assert len(recent) == 3
        # 应是最后 3 个
        assert recent[0].tool_name == "file_edit"
        # args_hash 应该是 a7/a8/a9
        assert "a7" not in recent[0].args_hash  # 仅检查长度即可

    def test_reset_clears_all(self):
        tracker = ToolCallTracker()
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.reset()
        assert tracker.total_count() == 0
        assert tracker.consecutive_errors() == 0
        assert tracker.is_window_all_failed(3) is False

    def test_total_count(self):
        tracker = ToolCallTracker()
        for i in range(5):
            tracker.record("file_edit", {"path": f"a{i}.py"}, True)
        assert tracker.total_count() == 5


# ---------------- FIFO 淘汰 ----------------

class TestFIFOEviction:
    def test_max_history_default_100(self):
        """默认 max_history=100，超过时 FIFO 淘汰。"""
        tracker = ToolCallTracker(max_history=5)
        for i in range(10):
            tracker.record("file_edit", {"path": f"a{i}.py"}, True)
        # 仅保留最后 5 条
        assert tracker.total_count() == 5
        recent = tracker.recent(10)
        # 第一条应该是 a5（0-4 被淘汰）
        # 通过 args_hash 验证（仅检查数量足够）
        assert len(recent) == 5

    def test_fifo_does_not_affect_consecutive_errors(self):
        """FIFO 淘汰不应重置 consecutive_errors。"""
        tracker = ToolCallTracker(max_history=3)
        for i in range(5):
            tracker.record("file_edit", {"path": f"a{i}.py"}, False, error="err")
        # 5 次失败但 max_history=3，consecutive_errors 应仍是 5
        assert tracker.consecutive_errors() == 5
        assert tracker.total_count() == 3  # 历史被淘汰到 3


# ---------------- get_status() ----------------

class TestGetStatus:
    def test_empty_status(self):
        tracker = ToolCallTracker()
        status = tracker.get_status()
        assert status["total_calls"] == 0
        assert status["success_count"] == 0
        assert status["fail_count"] == 0
        assert status["consecutive_errors"] == 0
        assert status["window_all_failed"] is False
        assert status["same_error_repeated"] is False

    def test_mixed_status(self):
        tracker = ToolCallTracker()
        tracker.record("file_read", {"path": "a.py"}, True)
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        tracker.record("file_edit", {"path": "a.py"}, False, error="err")
        status = tracker.get_status()
        assert status["total_calls"] == 3
        assert status["success_count"] == 1
        assert status["fail_count"] == 2
        assert status["consecutive_errors"] == 2
        # 3 次中只有 2 次失败，window_all_failed=False
        assert status["window_all_failed"] is False


# ---------------- 默认单例 ----------------

class TestDefaultSingleton:
    def setup_method(self):
        reset_default_tracker()

    def teardown_method(self):
        reset_default_tracker()

    def test_get_default_tracker_returns_same_instance(self):
        t1 = get_default_tracker()
        t2 = get_default_tracker()
        assert t1 is t2

    def test_reset_default_tracker_creates_new(self):
        t1 = get_default_tracker()
        reset_default_tracker()
        t2 = get_default_tracker()
        assert t1 is not t2

    def test_default_tracker_is_usable(self):
        tracker = get_default_tracker()
        tracker.record("file_read", {"path": "a.py"}, True)
        assert tracker.total_count() == 1
        assert tracker.consecutive_errors() == 0


# ---------------- 线程安全 ----------------

class TestThreadSafety:
    def test_concurrent_records_no_loss(self):
        """并发记录不应丢失（线程安全）。"""
        tracker = ToolCallTracker(max_history=1000)
        N_THREADS = 10
        N_PER_THREAD = 50

        def worker(tid):
            for i in range(N_PER_THREAD):
                tracker.record(
                    "file_edit", {"tid": tid, "i": i}, True
                )

        threads = [threading.Thread(target=worker, args=(t,)) for t in range(N_THREADS)]
        for t in threads:
            t.start()
        for t in threads:
            t.join()

        assert tracker.total_count() == N_THREADS * N_PER_THREAD

    def test_concurrent_window_check_safe(self):
        """并发调用 is_window_all_failed 不应抛异常。"""
        tracker = ToolCallTracker()
        for i in range(5):
            tracker.record("file_edit", {"path": f"a{i}.py"}, False, error="err")

        errors = []

        def worker():
            try:
                for _ in range(100):
                    tracker.is_window_all_failed(3)
                    tracker.is_same_error_repeated(3)
                    tracker.consecutive_errors()
                    tracker.recent(5)
                    tracker.get_status()
            except Exception as e:
                errors.append(e)

        threads = [threading.Thread(target=worker) for _ in range(5)]
        for t in threads:
            t.start()
        for t in threads:
            t.join()
        assert errors == []



# -*- coding: utf-8 -*-
"""curator 模块测试"""
import json
import tempfile
from datetime import datetime, timedelta, timezone
from pathlib import Path

import pytest

from abyss.curator import (
    STATE_ACTIVE,
    STATE_STALE,
    STATE_ARCHIVED,
    bump_use,
    load_usage,
    save_usage,
    load_state,
    save_state,
    should_run_now,
    apply_automatic_transitions,
    pin_skill,
    unpin_skill,
    restore_skill,
    list_archived,
    get_report,
)


class TestUsageTracking:
    """使用记录跟踪测试"""

    def test_bump_use_creates_new_record(self, monkeypatch, tmp_path):
        """首次使用创建记录。"""
        from abyss.curator import _usage_file

        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        bump_use("test-skill")
        data = load_usage()
        assert "test-skill" in data
        assert data["test-skill"]["use_count"] == 1
        assert data["test-skill"]["state"] == STATE_ACTIVE
        assert data["test-skill"]["pinned"] is False

    def test_bump_use_increments_count(self, monkeypatch, tmp_path):
        """多次使用累计计数。"""
        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        bump_use("test-skill")
        bump_use("test-skill")
        bump_use("test-skill")
        data = load_usage()
        assert data["test-skill"]["use_count"] == 3

    def test_bump_use_reactivates_stale(self, monkeypatch, tmp_path):
        """使用过期 skill 时重新激活。"""
        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        # 先手动创建为 stale
        save_usage({"test-skill": {"use_count": 0, "state": STATE_STALE, "pinned": False}})
        bump_use("test-skill")
        data = load_usage()
        assert data["test-skill"]["state"] == STATE_ACTIVE

    def test_bump_use_empty_name(self, monkeypatch, tmp_path):
        """空名称静默忽略。"""
        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)
        bump_use("")  # 不应抛异常
        bump_use(None)  # 不应抛异常


class TestStateFile:
    """状态文件读写测试"""

    def test_load_state_default(self, monkeypatch, tmp_path):
        state_path = tmp_path / ".curator_state"
        monkeypatch.setattr("abyss.curator._state_file", lambda: state_path)
        state = load_state()
        assert state["paused"] is False
        assert state["run_count"] == 0
        assert state["last_run_at"] is None

    def test_save_and_load_state(self, monkeypatch, tmp_path):
        state_path = tmp_path / ".curator_state"
        monkeypatch.setattr("abyss.curator._state_file", lambda: state_path)
        save_state({"last_run_at": "2026-01-01T00:00:00", "paused": True, "run_count": 5})
        state = load_state()
        assert state["paused"] is True
        assert state["run_count"] == 5
        assert state["last_run_at"] == "2026-01-01T00:00:00"


class TestShouldRunNow:
    """运行条件检查测试"""

    def test_first_run_defers(self, monkeypatch, tmp_path):
        """首次调用不运行，记录时间。"""
        state_path = tmp_path / ".curator_state"
        monkeypatch.setattr("abyss.curator._state_file", lambda: state_path)
        now = datetime(2026, 7, 1, 12, 0, 0, tzinfo=timezone.utc)
        assert should_run_now(now=now) is False

        state = load_state()
        assert state["last_run_at"] is not None

    def test_before_interval(self, monkeypatch, tmp_path):
        """未到 interval 不运行。"""
        state_path = tmp_path / ".curator_state"
        monkeypatch.setattr("abyss.curator._state_file", lambda: state_path)

        save_state({"last_run_at": "2026-07-01T12:00:00+00:00", "paused": False, "run_count": 1})
        now = datetime(2026, 7, 1, 23, 0, 0, tzinfo=timezone.utc)
        assert should_run_now(now=now) is False

    def test_after_interval_runs(self, monkeypatch, tmp_path):
        """超过 interval 后运行。"""
        state_path = tmp_path / ".curator_state"
        monkeypatch.setattr("abyss.curator._state_file", lambda: state_path)

        save_state({"last_run_at": "2026-07-01T12:00:00+00:00", "paused": False, "run_count": 1})
        now = datetime(2026, 7, 3, 12, 0, 0, tzinfo=timezone.utc)
        assert should_run_now(now=now) is True

    def test_paused_skips(self, monkeypatch, tmp_path):
        """暂停时跳过。"""
        state_path = tmp_path / ".curator_state"
        monkeypatch.setattr("abyss.curator._state_file", lambda: state_path)

        save_state({"last_run_at": "2026-06-01T12:00:00+00:00", "paused": True, "run_count": 1})
        now = datetime(2026, 7, 6, 12, 0, 0, tzinfo=timezone.utc)
        assert should_run_now(now=now) is False


class TestAutomaticTransitions:
    """自动状态过渡测试"""

    @pytest.fixture
    def skill_env(self, monkeypatch, tmp_path):
        """准备测试用 skill 目录和环境。"""
        skills_dir = tmp_path / "skills"
        skills_dir.mkdir()
        usage_path = skills_dir / ".usage.json"

        monkeypatch.setattr("abyss.curator._skills_dir", lambda: skills_dir)
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)
        monkeypatch.setattr("abyss.curator._archive_dir", lambda: skills_dir / ".archive")

        # 创建测试 skill
        def make_skill(dirname, name, days_ago=0):
            d = skills_dir / dirname
            d.mkdir()
            mtime = (datetime.now() - timedelta(days=days_ago)).timestamp()
            (d / "SKILL.md").write_text(
                f"---\nname: {name}\ndescription: Test\n---\n# {name}\n",
                encoding="utf-8",
            )
            # 设目录时间（apply_automatic_transitions 用 skill_dir.stat()）
            import os
            os.utime(str(d), (mtime, mtime))

        return skills_dir, make_skill

    def test_apply_stale(self, skill_env):
        """超过 stale 天数未使用 → 标记 stale。"""
        dirs, make = skill_env
        make("old-skill", "old-skill", days_ago=30)

        now = datetime.now(timezone.utc)
        from abyss.curator import DEFAULT_STALE_AFTER_DAYS
        counts = apply_automatic_transitions(now=now)
        assert counts["checked"] == 1
        assert counts["marked_stale"] == 1

        usage = load_usage()
        assert usage["old-skill"]["state"] == STATE_STALE

    def test_pinned_bypasses(self, skill_env):
        """固定 skill 不受影响。"""
        dirs, make = skill_env
        make("old-skill", "old-skill", days_ago=60)

        # 先固定
        save_usage({"old-skill": {"pinned": True, "use_count": 0, "state": STATE_ACTIVE}})

        now = datetime.now(timezone.utc)
        counts = apply_automatic_transitions(now=now)
        assert counts["marked_stale"] == 0
        assert counts["archived"] == 0


class TestPinUnpin:
    """固定/取消固定测试"""

    def test_pin(self, monkeypatch, tmp_path):
        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        assert pin_skill("my-skill") is True
        usage = load_usage()
        assert usage["my-skill"]["pinned"] is True

    def test_unpin(self, monkeypatch, tmp_path):
        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        save_usage({"my-skill": {"pinned": True, "state": STATE_ACTIVE, "use_count": 0}})
        assert unpin_skill("my-skill") is True
        usage = load_usage()
        assert usage["my-skill"]["pinned"] is False


class TestRestore:
    """归档恢复测试"""

    def test_restore_not_archived(self, monkeypatch, tmp_path):
        """未归档 skill 无法恢复。"""
        usage_path = tmp_path / ".usage.json"
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        assert restore_skill("nonexistent") is False


class TestGetReport:
    """报告生成测试"""

    def test_report_empty(self, monkeypatch, tmp_path):
        """无 skill 时返回空列表。"""
        skills_dir = tmp_path / "skills"
        skills_dir.mkdir()
        usage_path = skills_dir / ".usage.json"
        monkeypatch.setattr("abyss.curator._skills_dir", lambda: skills_dir)
        monkeypatch.setattr("abyss.curator._usage_file", lambda: usage_path)

        report = get_report()
        assert report == []



# -*- coding: utf-8 -*-
"""FileStateCache LRU 文件状态缓存（Phase 6 Tier 1-4 AQ）。

借鉴 zhikuncode-main FileStateCache.java：
- OrderedDict accessOrder=true 实现 LRU
- 大小限制：100 条 / 25MB 任一超限淘汰
- 路径规范化（os.path.realpath）作为 key
- 存储 (mtime_ns, size, content) 三元组
- 读取时比对 mtime_ns + size，未变则命中
"""
from collections import OrderedDict
import os
import threading
from typing import Optional, Tuple

__all__ = ["FileStateCache", "get_default_cache"]


class FileStateCache:
    """LRU 文件状态缓存。

    - 路径规范化（os.path.realpath）作为 key
    - 存储 (mtime_ns, size, content) 三元组
    - 读取时比对 mtime_ns + size，未变则命中；否则淘汰并返回 None
    - 任一超限（max_entries 或 max_total_bytes）触发 LRU 淘汰
    """

    MAX_ENTRIES = 100
    MAX_TOTAL_BYTES = 25 * 1024 * 1024  # 25MB

    def __init__(self, max_entries: Optional[int] = None, max_total_bytes: Optional[int] = None):
        # 允许子类/测试覆盖默认值
        if max_entries is not None:
            self.MAX_ENTRIES = max_entries
        if max_total_bytes is not None:
            self.MAX_TOTAL_BYTES = max_total_bytes
        self._cache: "OrderedDict[str, Tuple[int, int, str]]" = OrderedDict()
        self._total_bytes = 0
        self._lock = threading.Lock()

    # ------------------------------------------------------------------ public API

    def get(self, path: str) -> Optional[str]:
        """读取缓存。命中返回内容，未命中或已失效返回 None。"""
        try:
            real = os.path.realpath(path)
        except OSError:
            return None
        try:
            st = os.stat(real)
        except OSError:
            # 文件不存在：若缓存里有则清理，否则直接返回 None
            with self._lock:
                if real in self._cache:
                    self._evict_locked(real)
            return None

        with self._lock:
            entry = self._cache.get(real)
            if entry is None:
                return None
            mtime_ns, size, content = entry
            if st.st_mtime_ns != mtime_ns or st.st_size != size:
                # 文件已被外部修改，失效
                self._evict_locked(real)
                return None
            # 命中：移到末尾（最近使用）
            self._cache.move_to_end(real)
            return content

    def put(self, path: str, content: str) -> None:
        """写入缓存。若文件不存在则忽略（无法记录 mtime）。"""
        try:
            real = os.path.realpath(path)
        except OSError:
            return
        try:
            st = os.stat(real)
        except OSError:
            # 文件不存在：不缓存（避免后续 get 误命中）
            return

        size = len(content.encode("utf-8"))

        with self._lock:
            # 若已存在，先移除旧条目
            if real in self._cache:
                self._evict_locked(real)
            # 容量预检 + 淘汰
            while (len(self._cache) >= self.MAX_ENTRIES or
                   (self._total_bytes + size > self.MAX_TOTAL_BYTES)):
                if not self._cache:
                    break
                self._evict_oldest_locked()
            # 写入
            self._cache[real] = (st.st_mtime_ns, st.st_size, content)
            self._total_bytes += size

    def invalidate(self, path: str) -> None:
        """显式失效某条目。不存在的路径不抛异常。"""
        try:
            real = os.path.realpath(path)
        except OSError:
            return
        with self._lock:
            if real in self._cache:
                self._evict_locked(real)

    def clear(self) -> None:
        """清空所有缓存。"""
        with self._lock:
            self._cache.clear()
            self._total_bytes = 0

    # ------------------------------------------------------------------ 内部方法（调用方已持锁）

    def _evict_locked(self, key: str) -> None:
        """移除指定 key（调用方持锁）。"""
        entry = self._cache.pop(key, None)
        if entry is not None:
            _, _, content = entry
            self._total_bytes -= len(content.encode("utf-8"))
            if self._total_bytes < 0:
                self._total_bytes = 0

    def _evict_oldest_locked(self) -> None:
        """淘汰最久未使用条目（OrderedDict 首项，调用方持锁）。"""
        if not self._cache:
            return
        key, entry = self._cache.popitem(last=False)
        _, _, content = entry
        self._total_bytes -= len(content.encode("utf-8"))
        if self._total_bytes < 0:
            self._total_bytes = 0


# ------------------------------------------------------------------ 模块级单例

_default_cache: Optional[FileStateCache] = None
_default_lock = threading.Lock()


def get_default_cache() -> FileStateCache:
    """返回模块级默认 FileStateCache 单例。"""
    global _default_cache
    if _default_cache is None:
        with _default_lock:
            if _default_cache is None:
                _default_cache = FileStateCache()
    return _default_cache



# -*- coding: utf-8 -*-
"""SelfCorrectionLoop 自纠错循环测试。

借鉴 zhikuncode-main 的 SelfCorrectionLoop：工具执行失败后生成
结构化再提示供 LLM 修正参数，最多重试 max_attempts 次，提升 bug 修复成功率。

测试只验证 SelfCorrectionLoop 类本身逻辑，不依赖真实 LLM 调用。
"""
import pytest

from abyss.agent_loop import SelfCorrectionLoop


def make_tool_call(name: str, args: str = "{}") -> dict:
    """构造 OpenAI 格式 tool_call mock。"""
    return {
        "id": f"call_{name}",
        "type": "function",
        "function": {"name": name, "arguments": args},
    }


def make_failure(error: str = "未找到匹配的旧内容") -> dict:
    """构造失败结果。"""
    return {"success": False, "error": error}


def make_success() -> dict:
    """构造成功结果。"""
    return {"success": True, "output": "ok"}


class TestSelfCorrectionLoopBasic:
    """SelfCorrectionLoop 基本行为。"""

    def test_initial_attempts_zero(self):
        """初始化时 attempts 为 0。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        assert loop.attempts == 0

    def test_should_retry_on_failure(self):
        """工具失败时 should_retry 返回 True。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure()
        assert loop.should_retry(tc, result) is True

    def test_should_not_retry_on_success(self):
        """工具成功时 should_retry 返回 False。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_success()
        assert loop.should_retry(tc, result) is False


class TestBuildCorrectionPrompt:
    """build_correction_prompt 生成结构化提示。"""

    def test_contains_tool_name_and_error(self):
        """提示包含工具名和失败原因。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure("未找到匹配的旧内容")
        prompt = loop.build_correction_prompt(tc, result, attempt=1)
        assert "file_edit" in prompt
        assert "未找到匹配的旧内容" in prompt

    def test_file_edit_suggestion(self):
        """file_edit 失败建议先 file_read 确认内容。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure()
        prompt = loop.build_correction_prompt(tc, result, attempt=1)
        assert "file_read" in prompt

    def test_shell_exec_suggestion(self):
        """shell_exec 失败建议检查命令语法。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("shell_exec")
        result = make_failure()
        prompt = loop.build_correction_prompt(tc, result, attempt=1)
        assert "检查命令语法" in prompt

    def test_generic_suggestion(self):
        """通用工具失败包含根据错误信息调整参数。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("unknown_tool")
        result = make_failure()
        prompt = loop.build_correction_prompt(tc, result, attempt=1)
        assert "根据错误信息调整参数" in prompt


class TestFailureTriggersRetry:
    """失败触发重试。"""

    def test_failure_increments_attempts(self):
        """失败一次后 attempts 计为 1。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure()
        loop.should_retry(tc, result)
        assert loop.attempts == 1

    def test_should_retry_below_max(self):
        """attempts < max_attempts 时返回 True。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure()
        # 第一次失败
        assert loop.should_retry(tc, result) is True
        assert loop.attempts == 1
        # 第二次失败
        assert loop.should_retry(tc, result) is True
        assert loop.attempts == 2

    def test_correction_prompt_after_failure(self):
        """失败后能生成下一轮提示。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure()
        loop.should_retry(tc, result)
        prompt = loop.build_correction_prompt(tc, result, attempt=loop.attempts)
        assert "file_edit" in prompt


class TestSuccessStopsLoop:
    """成功后立即停止。"""

    def test_success_does_not_retry(self):
        """第 1 次重试成功后不再重试。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        fail = make_failure()
        success = make_success()
        # 失败一次
        assert loop.should_retry(tc, fail) is True
        # 成功
        assert loop.should_retry(tc, success) is False
        # attempts 不再增长
        assert loop.attempts == 1


class TestExhaustedLoop:
    """达到上限放弃。"""

    def test_max_attempts_exhausted(self):
        """连续失败 3 次后 should_retry 返回 False。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        result = make_failure()
        # 3 次失败均允许重试
        assert loop.should_retry(tc, result) is True  # attempt 1
        assert loop.should_retry(tc, result) is True  # attempt 2
        assert loop.should_retry(tc, result) is True  # attempt 3
        assert loop.attempts == 3
        # 第 4 次不应再重试
        assert loop.should_retry(tc, result) is False

    def test_exhausted_message(self):
        """build_exhausted_message 返回耗尽提示。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        msg = loop.build_exhausted_message()
        assert "自纠错循环已耗尽 3 次重试" in msg


class TestOnlyFailureTriggers:
    """仅对工具失败触发重试。"""

    def test_success_result_no_retry(self):
        """成功结果不触发重试。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        # 各种成功形式
        assert loop.should_retry(tc, {"success": True, "output": "ok"}) is False
        assert loop.should_retry(tc, {"success": True}) is False
        # 无 success 字段默认视为成功
        assert loop.should_retry(tc, {"output": "ok"}) is False

    def test_failure_result_triggers_retry(self):
        """失败结果触发重试。"""
        loop = SelfCorrectionLoop(max_attempts=3)
        tc = make_tool_call("file_edit")
        assert loop.should_retry(tc, {"success": False, "error": "失败"}) is True
        assert loop.should_retry(tc, {"success": False, "error": "another"}) is True



# -*- coding: utf-8 -*-
"""
ansi_menu 视觉辅助函数测试：
- ansi_green_bold / ansi_cyan / ansi_gray_line
- ansi_prompt 的 prompt_color 参数
"""
import os
from unittest.mock import patch

import pytest

from abyss.ansi_menu import (
    ansi_green_bold,
    ansi_cyan,
    ansi_gray_line,
    ansi_prompt,
)


# ─── 颜色包装函数 ────────────────────────────────────────
class TestAnsiColorHelpers:
    def test_ansi_green_bold_wraps_with_codes(self):
        s = ansi_green_bold(">")
        assert "\x1b[1;32m" in s
        assert "\x1b[0m" in s
        assert ">" in s

    def test_ansi_green_bold_preserves_content(self):
        s = ansi_green_bold("hello")
        # 去掉所有 ANSI 转义后应等于原内容
        import re
        plain = re.sub(r"\x1b\[[0-9;]*m", "", s)
        assert plain == "hello"

    def test_ansi_cyan_wraps_with_codes(self):
        s = ansi_cyan("text")
        assert "\x1b[36m" in s
        assert "\x1b[0m" in s
        assert "text" in s

    def test_ansi_cyan_preserves_content(self):
        s = ansi_cyan("world")
        import re
        plain = re.sub(r"\x1b\[[0-9;]*m", "", s)
        assert plain == "world"


# ─── 灰色分隔线 ──────────────────────────────────────────
class TestAnsiGrayLine:
    def test_default_contains_gray_code_and_horizontal_char(self):
        line = ansi_gray_line()
        assert "\x1b[90m" in line
        assert "\x1b[0m" in line
        assert "─" in line

    def test_default_length_within_bounds(self):
        """默认宽度应在 [20, 60] 之间。"""
        line = ansi_gray_line()
        # 计算实际 ─ 的数量
        count = line.count("─")
        assert 20 <= count <= 60

    def test_custom_width(self):
        line = ansi_gray_line(width=15)
        assert line.count("─") == 15

    def test_zero_or_negative_width_falls_back_to_auto(self):
        """width<=0 时自动取终端宽度（夹在 20-60）。"""
        line = ansi_gray_line(width=0)
        count = line.count("─")
        assert 20 <= count <= 60

    def test_terminal_width_capped_at_60(self):
        """终端很宽时分隔线不超过 60。"""
        with patch("os.get_terminal_size") as mock_ts:
            mock_ts.return_value = os.terminal_size((200, 50))
            line = ansi_gray_line()
            assert line.count("─") == 60

    def test_terminal_width_floored_at_20(self):
        """终端很窄时分隔线至少 20。"""
        with patch("os.get_terminal_size") as mock_ts:
            mock_ts.return_value = os.terminal_size((10, 5))
            line = ansi_gray_line()
            assert line.count("─") == 20

    def test_terminal_size_failure_falls_back_to_80(self):
        """os.get_terminal_size 抛异常时回退到 80 列（min(60, 80)=60）。"""
        with patch("os.get_terminal_size", side_effect=OSError):
            line = ansi_gray_line()
            assert line.count("─") == 60


# ─── ansi_prompt prompt_color 参数 ──────────────────────
class TestAnsiPromptColor:
    """ansi_prompt 接受 prompt_color 参数：渲染时包裹颜色，buffer 仍存纯文本，
    返回值不含任何 ANSI 码。"""

    def _feed_keys(self, keys):
        """构造一个 msvcrt.getwch 的 mock，依次返回 keys 中每个字符。"""
        seq = list(keys)
        import itertools
        counter = {"i": 0}

        def fake_getwch():
            if counter["i"] >= len(seq):
                # 防止无限循环：持续返回 \r
                return "\r"
            ch = seq[counter["i"]]
            counter["i"] += 1
            return ch
        return fake_getwch

    def test_prompt_color_renders_colored_prefix(self, capsys):
        """传入 prompt_color，stdout 应包含颜色化的提示符。"""
        keys = "hi\r"
        with patch("msvcrt.getwch", side_effect=self._feed_keys(keys)):
            with patch("msvcrt.kbhit", return_value=False):
                result = ansi_prompt("> ", prompt_color="\x1b[1;32m")
        out = capsys.readouterr().out
        # 渲染时应包含绿色粗体码 + 提示符 + 复位码
        assert "\x1b[1;32m> \x1b[0m" in out or "\x1b[1;32m>" in out
        # 返回值不含 ANSI 码
        assert result == "hi"

    def test_prompt_color_return_excludes_color(self, capsys):
        """返回值应为纯用户输入，不含 ANSI 码。"""
        keys = "hello\r"
        with patch("msvcrt.getwch", side_effect=self._feed_keys(keys)):
            with patch("msvcrt.kbhit", return_value=False):
                result = ansi_prompt("> ", prompt_color="\x1b[1;32m")
        assert "\x1b" not in result
        assert result == "hello"

    def test_no_color_default_behavior(self, capsys):
        """不传 prompt_color 时，行为与原版一致（无颜色码）。"""
        keys = "ok\r"
        with patch("msvcrt.getwch", side_effect=self._feed_keys(keys)):
            with patch("msvcrt.kbhit", return_value=False):
                result = ansi_prompt("> ")
        out = capsys.readouterr().out
        assert result == "ok"
        # 默认不传颜色时，stdout 不应包含绿色码
        assert "\x1b[1;32m" not in out



# -*- coding: utf-8 -*-
"""agent_loop 模块测试"""
import json
from unittest.mock import MagicMock

import pytest

from abyss.agent_loop import (
    _extract_missing_param,
    execute_tool,
    execute_tool_batch,
    process_tool_calls,
)
from abyss.tools.registry import ToolRegistry, format_tool_result
from abyss.tools.base import Tool


class FakeTool(Tool):
    """测试用 Fake Tool，echo 返回参数。"""
    @property
    def name(self) -> str:
        return "fake_echo"

    @property
    def description(self) -> str:
        return "Echo back the message"

    @property
    def parameters(self) -> dict:
        return {
            "type": "object",
            "properties": {
                "message": {"type": "string", "description": "Message to echo"},
            },
            "required": ["message"],
        }

    def execute(self, message: str, **kwargs) -> dict:
        return {"success": True, "echo": message}


class TestExtractMissingParam:
    """_extract_missing_param 工具函数测试"""

    def test_extract_named_param(self):
        """从 TypeError 提取具名参数。"""
        exc = TypeError("FakeTool.execute() missing 1 required positional argument: 'message'")
        result = _extract_missing_param(exc, lambda message: None)
        assert result == "message"

    def test_extract_generic_missing(self):
        """无具名参数时列函数签名。"""
        exc = TypeError("missing 2 required positional arguments")
        result = _extract_missing_param(exc, lambda a, b, c=1: None)
        assert result == "a, b"

    def test_no_match(self):
        """不匹配任何模式返回空串。"""
        exc = TypeError("unexpected keyword argument 'foo'")
        result = _extract_missing_param(exc, lambda: None)
        assert result == ""


class TestExecuteTool:
    """execute_tool 核心逻辑测试"""

    def make_registry(self):
        reg = ToolRegistry()
        reg.register(FakeTool())
        return reg

    def test_dispatches_to_tool(self):
        """正确派发到已注册工具。"""
        reg = self.make_registry()
        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": json.dumps({"message": "hello"}),
            }
        }
        result = json.loads(execute_tool(tc, reg))
        assert result["success"] is True
        assert result["echo"] == "hello"

    def test_unknown_tool(self):
        """未知工具返回错误。"""
        reg = self.make_registry()
        tc = {
            "function": {
                "name": "no_such_tool",
                "arguments": "{}",
            }
        }
        result = json.loads(execute_tool(tc, reg))
        assert result["success"] is False
        assert "未知工具" in result["error"]

    def test_bad_json_arguments(self):
        """参数 JSON 解析失败返回错误。"""
        reg = self.make_registry()
        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": "not json",
            }
        }
        result = json.loads(execute_tool(tc, reg))
        assert result["success"] is False
        assert "参数解析失败" in result["error"]

    def test_missing_required_param(self):
        """缺必要参数时返回结构化错误，不抛 TypeError。"""
        reg = self.make_registry()
        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": "{}",
            }
        }
        result = json.loads(execute_tool(tc, reg))
        assert result["success"] is False
        assert "缺失必要参数" in result["error"]

    def test_pre_tool_hook_blocks(self):
        """PreToolUse 钩子阻断工具调用。"""
        reg = self.make_registry()
        host = MagicMock()
        host.pre_tool.return_value = {"block": True, "reason": "test block"}

        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": json.dumps({"message": "hello"}),
            }
        }
        result = json.loads(execute_tool(tc, reg, host=host))
        assert result["success"] is False
        assert "被钩子阻断" in result["error"]
        assert "test block" in result["error"]

    def test_pre_tool_hook_allows(self):
        """PreToolUse 钩子不阻断时正常执行。"""
        reg = self.make_registry()
        host = MagicMock()
        host.pre_tool.return_value = {"block": False}

        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": json.dumps({"message": "hello"}),
            }
        }
        result = json.loads(execute_tool(tc, reg, host=host))
        assert result["success"] is True

    def test_no_host_nohook(self):
        """host 为 None 时跳过钩子，正常执行。"""
        reg = self.make_registry()
        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": json.dumps({"message": "hello"}),
            }
        }
        result = json.loads(execute_tool(tc, reg, host=None))
        assert result["success"] is True

    def test_post_tool_call(self):
        """PostToolUse 钩子在执行后被调用。"""
        reg = self.make_registry()
        host = MagicMock()
        host.pre_tool.return_value = {"block": False}
        host.post_tool = MagicMock()

        tc = {
            "function": {
                "name": "fake_echo",
                "arguments": json.dumps({"message": "hello"}),
            }
        }
        execute_tool(tc, reg, host=host)
        host.post_tool.assert_called_once()


class TestExecuteToolBatch:
    """execute_tool_batch 测试"""

    def test_runs_all_tools(self):
        """批量执行所有工具调用。"""
        reg = ToolRegistry()
        reg.register(FakeTool())
        tcs = [
            {"function": {"name": "fake_echo", "arguments": json.dumps({"message": "a"})}},
            {"function": {"name": "fake_echo", "arguments": json.dumps({"message": "b"})}},
        ]
        results = execute_tool_batch(1, tcs, reg)
        assert len(results) == 2
        assert json.loads(results[0])["echo"] == "a"
        assert json.loads(results[1])["echo"] == "b"


class TestProcessToolCalls:
    """process_tool_calls 循环逻辑测试"""

    def make_session(self):
        from abyss.session import Session
        return Session("You are a test assistant.")

    def make_registry(self):
        reg = ToolRegistry()
        reg.register(FakeTool())
        return reg

    def test_single_call_no_more_tools(self):
        """工具调用后 AI 不再要求更多工具 → 返回最终 content。"""
        session = self.make_session()
        reg = self.make_registry()
        client = MagicMock()

        # 模拟 stream_handler：返回内容无新 tool_calls
        def fake_stream(client, msgs, schema, cfg, sr):
            return ("done", "", [], None)

        tc = {"function": {"name": "fake_echo", "arguments": json.dumps({"message": "hi"})}}

        result = process_tool_calls(
            session=session,
            client=client,
            tools_schemas=reg.to_openai_schemas(),
            tool_calls=[tc],
            cfg={"tool_result_max_chars": None},
            registry=reg,
            stream_handler=fake_stream,
        )
        assert result == "done"

    def test_follow_up_tool_loops(self):
        """多轮工具调用循环正确。"""
        session = self.make_session()
        reg = self.make_registry()
        client = MagicMock()

        call_count = [0]

        def fake_stream(client, msgs, schema, cfg, sr):
            call_count[0] += 1
            if call_count[0] == 1:
                # 第 2 轮（首次 follow-up）：继续要工具
                new_tc = [
                    {"index": 0, "id": "call_2", "function": {"name": "fake_echo", "arguments": json.dumps({"message": "again"})}}
                ]
                return ("", "", new_tc, None)
            else:
                # 第 3 轮：最终回复
                return ("all done", "", [], None)

        tc = {"function": {"name": "fake_echo", "arguments": json.dumps({"message": "hi"})}}

        result = process_tool_calls(
            session=session,
            client=client,
            tools_schemas=reg.to_openai_schemas(),
            tool_calls=[tc],
            cfg={"tool_result_max_chars": None},
            registry=reg,
            stream_handler=fake_stream,
        )
        assert result == "all done"
        assert call_count[0] == 2

    def test_engine_usage_update(self):
        """上下文引擎接收 usage 更新。"""
        session = self.make_session()
        reg = self.make_registry()
        engine = MagicMock()

        def fake_stream(client, msgs, schema, cfg, sr):
            return ("done", "", [], {"prompt_tokens": 500, "total_tokens": 600})

        tc = {"function": {"name": "fake_echo", "arguments": json.dumps({"message": "hi"})}}

        process_tool_calls(
            session=session,
            client=MagicMock(),
            tools_schemas=reg.to_openai_schemas(),
            tool_calls=[tc],
            cfg={"tool_result_max_chars": None},
            registry=reg,
            stream_handler=fake_stream,
            engine=engine,
        )
        engine.update_from_response.assert_called_once_with({"prompt_tokens": 500, "total_tokens": 600})



# -*- coding: utf-8 -*-
"""
LLM 韧性套件：断路器 + 指数退避重试（Phase 6 AN）

借鉴 zhikuncode-main 的 ApiCircuitBreaker.java + RetryPolicy.java：
- 断路器三态机：CLOSED（正常） → OPEN（拒绝） → HALF_OPEN（探测） → CLOSED/HALF_OPEN
- 指数退避 + jitter：BASE_DELAY=500ms，最大 32s，抖动 0-25%
- 可重试错误分类：429/529/408/5xx/ECONNRESET 等

设计动机（abyss 当前状态）：
- src/abyss/api_client.py 仅有 RETRYABLE_CODES = {429, 500, 503} 简单集合
- 重试间隔固定 time.sleep(min(2 ** (attempt - 1), 8))，无 jitter
- 无断路器，连续失败时仍持续重试，可能加剧 API 雪崩
- 无 OPEN 态快速失败，浪费用户时间

与 Phase 3 I (LLM 韧性四件套) 的关系：本项提供熔断 + 重试的具体实现，
是 Phase 3 I 的前置基础。ApiKeyRotationManager / ModelDegradationChain 后续推进。
"""
import random
import threading
import time
from enum import Enum
from typing import Any, Callable, Optional, Set, Type

from openai import (
    APIError,
    APIConnectionError,
    APITimeoutError,
    InternalServerError,
    RateLimitError,
)


class CircuitState(Enum):
    """断路器三态枚举。

    - CLOSED：正常调用，失败计数达阈值（默认 5 次）转 OPEN
    - OPEN：直接拒绝请求（fast-fail），冷却时间后转 HALF_OPEN
    - HALF_OPEN：放一个探测请求，成功转 CLOSED，失败转 OPEN
    """

    CLOSED = "closed"
    OPEN = "open"
    HALF_OPEN = "half_open"


class CircuitOpenError(Exception):
    """断路器开启时抛出，表示请求被 fast-fail。

    调用方应向上抛此异常，让用户感知到"服务暂时不可用"，
    而非继续重试浪费时间。
    """

    def __init__(self, message: str = "API 断路器开启，请稍后重试"):
        super().__init__(message)


class ApiCircuitBreaker:
    """API 断路器：CLOSED → OPEN → HALF_OPEN → CLOSED 状态机。

    线程安全：所有状态变更使用 threading.Lock 保护。
    设计参考 Michael Nygard 的《Release It!》中的 Circuit Breaker 模式。

    使用示例：

        breaker = ApiCircuitBreaker(failure_threshold=5, recovery_timeout=60)

        if not breaker.allow_request():
            raise CircuitOpenError()

        try:
            result = api_call()
            breaker.record_success()
            return result
        except Exception as e:
            if classifier.is_retryable(e):
                breaker.record_failure()
            raise
    """

    def __init__(
        self,
        failure_threshold: int = 5,
        recovery_timeout: float = 60.0,
    ):
        """初始化断路器。

        Args:
            failure_threshold: 连续失败多少次后转 OPEN（默认 5）
            recovery_timeout: OPEN 态冷却时间（秒，默认 60）
        """
        self.failure_threshold = failure_threshold
        self.recovery_timeout = recovery_timeout

        self._state = CircuitState.CLOSED
        self._failure_count = 0
        self._last_failure_time: Optional[float] = None
        self._lock = threading.Lock()
        # HALF_OPEN 探测中标志：True 表示已放行一个探测请求，等待结果
        self._half_open_probe_in_flight = False

    @property
    def state(self) -> CircuitState:
        """当前状态（无锁读，CAS 保证可见性）。"""
        return self._state

    @property
    def failure_count(self) -> int:
        """当前失败计数（无锁读）。"""
        return self._failure_count

    def allow_request(self) -> bool:
        """是否允许请求通过。

        - CLOSED：始终允许
        - OPEN：冷却时间到则转 HALF_OPEN 并允许；否则拒绝
        - HALF_OPEN：仅允许第一个探测请求，后续拒绝直到结果回来

        Returns:
            True 表示允许调用，False 表示 fast-fail
        """
        with self._lock:
            if self._state == CircuitState.CLOSED:
                return True

            if self._state == CircuitState.OPEN:
                # 检查是否已过冷却期
                if self._last_failure_time is None:
                    # 防御性：不应该发生（OPEN 态必然有 last_failure_time）
                    self._state = CircuitState.HALF_OPEN
                    self._half_open_probe_in_flight = True
                    return True
                elapsed = time.time() - self._last_failure_time
                if elapsed >= self.recovery_timeout:
                    # 转 HALF_OPEN，放行探测请求
                    self._state = CircuitState.HALF_OPEN
                    self._half_open_probe_in_flight = True
                    return True
                return False  # fast-fail

            if self._state == CircuitState.HALF_OPEN:
                if self._half_open_probe_in_flight:
                    return False  # 探测中，拒绝其他请求
                # 探测已完成（不应到达此分支，但防御性放行）
                self._half_open_probe_in_flight = True
                return True

            return True  # 防御性

    def record_success(self) -> None:
        """记录一次成功调用。

        - CLOSED：重置失败计数
        - HALF_OPEN：关闭断路器，恢复正常
        - OPEN：不应该发生（success 仅在 allow_request 返回 True 后调用）
        """
        with self._lock:
            self._failure_count = 0
            self._half_open_probe_in_flight = False
            if self._state == CircuitState.HALF_OPEN:
                self._state = CircuitState.CLOSED
            # CLOSED 状态保持不变

    def record_failure(self) -> None:
        """记录一次失败调用。

        - CLOSED：失败计数 +1，达阈值转 OPEN
        - HALF_OPEN：失败，转回 OPEN
        - OPEN：刷新 last_failure_time（延长冷却期）
        """
        now = time.time()
        with self._lock:
            self._last_failure_time = now
            self._half_open_probe_in_flight = False

            if self._state == CircuitState.CLOSED:
                self._failure_count += 1
                if self._failure_count >= self.failure_threshold:
                    self._state = CircuitState.OPEN
            elif self._state == CircuitState.HALF_OPEN:
                # 探测失败，重新打开
                self._state = CircuitState.OPEN
                self._failure_count = self.failure_threshold  # 保留以备日志
            elif self._state == CircuitState.OPEN:
                # 已 OPEN，仅刷新 last_failure_time（已在上面赋值）
                pass

    def reset(self) -> None:
        """重置断路器到 CLOSED 初始状态（测试用）。"""
        with self._lock:
            self._state = CircuitState.CLOSED
            self._failure_count = 0
            self._last_failure_time = None
            self._half_open_probe_in_flight = False


class RetryPolicy:
    """指数退避 + jitter 重试策略。

    借鉴 zhikuncode-main RetryPolicy.java（104 行）：
    - BASE_DELAY = 500ms，每次翻倍
    - MAX_DELAY = 32s（封顶）
    - JITTER_RATIO = 0.25，抖动范围 [0, base * 0.25]
    - MAX_ATTEMPTS = 3（总尝试次数，含首次）

    使用示例：

        policy = RetryPolicy()
        for attempt in range(1, policy.max_attempts + 1):
            try:
                return api_call()
            except Exception as e:
                if not policy.should_retry(attempt) or not policy.is_retryable_exception(e):
                    raise
                time.sleep(policy.delay(attempt))
    """

    def __init__(
        self,
        base_delay: float = 0.5,
        max_delay: float = 32.0,
        jitter_ratio: float = 0.25,
        max_attempts: int = 3,
    ):
        """初始化重试策略。

        Args:
            base_delay: 首次重试基础延迟（秒）
            max_delay: 单次重试最大延迟（秒）
            jitter_ratio: 抖动比例 [0, 1]，0 表示无抖动
            max_attempts: 最大尝试次数（含首次调用）
        """
        self.BASE_DELAY = base_delay
        self.MAX_DELAY = max_delay
        self.JITTER_RATIO = jitter_ratio
        self.MAX_ATTEMPTS = max_attempts
        self._random = random.Random()

    # 兼容属性访问（与文档常量命名一致）
    @property
    def base_delay(self) -> float:
        return self.BASE_DELAY

    @property
    def max_delay(self) -> float:
        return self.MAX_DELAY

    @property
    def jitter_ratio(self) -> float:
        return self.JITTER_RATIO

    @property
    def max_attempts(self) -> int:
        return self.MAX_ATTEMPTS

    def delay(self, attempt: int) -> float:
        """计算第 attempt 次重试的延迟时间。

        Args:
            attempt: 当前尝试次数（1 表示首次重试，即第 2 次调用前）

        Returns:
            延迟秒数，含 jitter
        """
        # 指数退避：base * 2^(attempt-1)
        base = min(self.BASE_DELAY * (2 ** (attempt - 1)), self.MAX_DELAY)
        # jitter 在 [0, base * ratio] 范围内
        jitter = self._random.uniform(0, base * self.JITTER_RATIO)
        return base + jitter

    def should_retry(self, attempt: int) -> bool:
        """是否应该继续重试。

        Args:
            attempt: 当前已尝试次数（1 表示首次调用已失败）

        Returns:
            True 表示可以继续重试，False 表示已达上限
        """
        return attempt < self.MAX_ATTEMPTS

    def is_retryable_exception(self, exc: Exception) -> bool:
        """判断异常是否可重试（委托给 RetryableErrorClassifier）。"""
        return RetryableErrorClassifier.is_retryable(exc)


class RetryableErrorClassifier:
    """LLM 错误分类器：区分可重试与不可重试错误。

    借鉴 zhikuncode-main LlmErrorClassifier.java（~150 行）的核心分类逻辑：
    - 429 RateLimitError：可重试（需要 backoff）
    - 500/502/503 InternalServerError：可重试（服务端临时故障）
    - 529 Overloaded：可重试（服务过载，zhikuncode 529 源分类重试）
    - 408 RequestTimeout：可重试
    - 400/401/403/404：不可重试（客户端错误，重试无用）
    - APIConnectionError：可重试（网络抖动）
    - APITimeoutError：可重试（请求超时）
    """

    # 可重试 HTTP 状态码
    RETRYABLE_STATUS_CODES: Set[int] = {
        408,  # Request Timeout
        429,  # Too Many Requests
        500,  # Internal Server Error
        502,  # Bad Gateway
        503,  # Service Unavailable
        529,  # Server Overloaded (Anthropic 风格)
    }

    # 可重试异常类型
    RETRYABLE_EXCEPTION_TYPES = (
        APIConnectionError,
        APITimeoutError,
    )

    @classmethod
    def is_retryable(cls, exc: Exception) -> bool:
        """判断异常是否可重试。

        Args:
            exc: 待判断的异常

        Returns:
            True 表示可重试，False 表示快速失败不重试
        """
        # 1. 检查异常类型
        if isinstance(exc, cls.RETRYABLE_EXCEPTION_TYPES):
            return True

        # 2. 检查 status_code 属性（OpenAI SDK 的 APIStatusError 子类）
        status_code = getattr(exc, "status_code", None)
        if status_code is not None:
            try:
                return int(status_code) in cls.RETRYABLE_STATUS_CODES
            except (TypeError, ValueError):
                pass

        # 3. 显式处理 RateLimitError / InternalServerError（有 status_code）
        if isinstance(exc, (RateLimitError, InternalServerError)):
            return True

        return False


class LlmResilienceGate:
    """LLM 韧性网关：组合断路器 + 重试策略，提供统一的 call() 接口。

    使用示例：

        breaker = ApiCircuitBreaker()
        retry_policy = RetryPolicy()
        gate = LlmResilienceGate(breaker, retry_policy)

        try:
            result = gate.call(lambda: client.chat(messages, ...))
        except CircuitOpenError:
            # 断路器开启，提示用户稍后重试
            ...
        except Exception as e:
            # 重试耗尽后的最终异常
            ...
    """

    def __init__(
        self,
        breaker: ApiCircuitBreaker,
        retry_policy: RetryPolicy,
        classifier: Type[RetryableErrorClassifier] = RetryableErrorClassifier,
    ):
        self._breaker = breaker
        self._retry_policy = retry_policy
        self._classifier = classifier

    def call(self, func: Callable[[], Any]) -> Any:
        """执行函数，应用断路器 + 重试策略。

        Args:
            func: 无参数的可调用对象（已绑定了参数的 LLM 调用）

        Returns:
            函数返回值

        Raises:
            CircuitOpenError: 断路器开启，请求被 fast-fail
            Exception: 重试耗尽后的最终异常（原样抛出）
        """
        attempt = 0
        while True:
            attempt += 1

            # 1. 断路器检查
            if not self._breaker.allow_request():
                raise CircuitOpenError()

            # 2. 执行函数
            try:
                result = func()
                self._breaker.record_success()
                return result
            except Exception as exc:
                # 3. 判断是否可重试
                if not self._classifier.is_retryable(exc):
                    # 不可重试异常：不记录为断路器失败（避免污染计数）
                    # 但仍向上抛出
                    raise

                # 4. 可重试异常：记录断路器失败
                self._breaker.record_failure()

                # 5. 检查重试次数
                if not self._retry_policy.should_retry(attempt):
                    raise  # 重试耗尽，抛出最后异常

                # 6. 等待退避时间
                time.sleep(self._retry_policy.delay(attempt))
                # 循环继续下一次尝试



# -*- coding: utf-8 -*-
"""context_engine 模块测试"""
import pytest
from abyss.context_engine import (
    ContextEngine,
    SimpleTruncationEngine,
    create_engine,
)


class TestSimpleTruncationEngine:
    """SimpleTruncationEngine 测试"""

    def test_create_default(self):
        engine = SimpleTruncationEngine()
        assert engine.name == "simple"
        assert engine.context_length == 128000
        assert engine.threshold_tokens == int(128000 * 0.75)

    def test_create_custom(self):
        engine = SimpleTruncationEngine(
            context_length=64000,
            threshold_percent=0.8,
            protect_first_n=2,
            protect_last_n=4,
        )
        assert engine.context_length == 64000
        assert engine.threshold_tokens == int(64000 * 0.8)
        assert engine.protect_first_n == 2
        assert engine.protect_last_n == 4

    def test_update_from_response(self):
        engine = SimpleTruncationEngine()
        engine.update_from_response({
            "prompt_tokens": 50000,
            "completion_tokens": 2000,
            "total_tokens": 52000,
        })
        assert engine.last_prompt_tokens == 50000
        assert engine.last_completion_tokens == 2000
        assert engine.last_total_tokens == 52000

    def test_should_compress_below_threshold(self):
        engine = SimpleTruncationEngine(context_length=128000, threshold_percent=0.75)
        engine.update_from_response({"prompt_tokens": 50000})
        assert engine.should_compress() is False

    def test_should_compress_above_threshold(self):
        engine = SimpleTruncationEngine(context_length=128000, threshold_percent=0.75)
        engine.update_from_response({"prompt_tokens": 100000})
        assert engine.should_compress() is True

    def test_should_compress_explicit_tokens(self):
        engine = SimpleTruncationEngine(context_length=128000, threshold_percent=0.75)
        assert engine.should_compress(prompt_tokens=96000) is True
        assert engine.should_compress(prompt_tokens=10000) is False

    def test_compress_preserves_system_prompt(self):
        engine = SimpleTruncationEngine(protect_last_n=2)
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
        ]
        result = engine.compress(messages)
        assert result[0] == messages[0]

    def test_compress_small_list_no_change(self):
        """消息数太少时不压缩。"""
        engine = SimpleTruncationEngine(protect_last_n=8)
        messages = [
            {"role": "system", "content": "sys"},
            {"role": "user", "content": "hello"},
            {"role": "assistant", "content": "hi"},
        ]
        result = engine.compress(messages)
        assert result == messages

    def test_compress_truncates_middle(self):
        engine = SimpleTruncationEngine(
            protect_first_n=1,
            protect_last_n=2,
        )
        messages = [
            {"role": "system", "content": "sys"},
            {"role": "user", "content": "msg1"},
            {"role": "assistant", "content": "msg2"},
            {"role": "user", "content": "msg3"},
            {"role": "assistant", "content": "msg4"},
            {"role": "user", "content": "msg5"},
            {"role": "assistant", "content": "msg6"},
        ]
        result = engine.compress(messages)
        # system + head(1) + truncation_note + tail(2)
        assert len(result) == 5
        assert result[0]["role"] == "system"
        assert result[1] == messages[1]  # head
        assert result[2]["role"] == "user"
        assert "上下文压缩" in result[2]["content"]
        assert result[3] == messages[5]  # tail[-2]
        assert result[4] == messages[6]  # tail[-1]

    def test_compress_increments_counter(self):
        engine = SimpleTruncationEngine(protect_first_n=1, protect_last_n=1)
        messages = [
            {"role": "system", "content": "sys"},
            {"role": "user", "content": "a"},
            {"role": "assistant", "content": "b"},
            {"role": "user", "content": "c"},
            {"role": "assistant", "content": "d"},
            {"role": "user", "content": "e"},
        ]
        engine.compress(messages)
        assert engine.compression_count == 1
        engine.compress(messages)
        assert engine.compression_count == 2

    def test_compress_with_focus_topic(self):
        engine = SimpleTruncationEngine(protect_first_n=1, protect_last_n=1)
        messages = [
            {"role": "system", "content": "sys"},
            {"role": "user", "content": "a"},
            {"role": "assistant", "content": "b"},
            {"role": "user", "content": "c"},
            {"role": "assistant", "content": "d"},
            {"role": "user", "content": "e"},
        ]
        result = engine.compress(messages, focus_topic="数据库迁移")
        assert "数据库迁移" in result[2]["content"]

    def test_compress_does_not_modify_original(self):
        engine = SimpleTruncationEngine(protect_last_n=1)
        messages = [
            {"role": "system", "content": "sys"},
            {"role": "user", "content": "a"},
            {"role": "assistant", "content": "b"},
            {"role": "user", "content": "c"},
        ]
        original = [dict(m) for m in messages]
        engine.compress(messages)
        assert messages == original  # 原始未被修改

    def test_update_model(self):
        engine = SimpleTruncationEngine(context_length=128000)
        engine.update_model(64000)
        assert engine.context_length == 64000
        assert engine.threshold_tokens == int(64000 * 0.75)

    def test_get_status(self):
        engine = SimpleTruncationEngine(context_length=128000)
        engine.update_from_response({"prompt_tokens": 96000})
        status = engine.get_status()
        assert status["last_prompt_tokens"] == 96000
        assert status["context_length"] == 128000
        assert status["usage_percent"] == 75
        assert status["compression_count"] == 0

    def test_on_session_reset(self):
        engine = SimpleTruncationEngine()
        engine.update_from_response({"prompt_tokens": 96000})
        engine.compression_count = 3
        engine.on_session_reset()
        assert engine.last_prompt_tokens == 0
        assert engine.compression_count == 0


class TestCreateEngine:
    """工厂函数测试"""

    def test_create_simple(self):
        engine = create_engine("simple", context_length=64000)
        assert isinstance(engine, SimpleTruncationEngine)
        assert engine.name == "simple"

    def test_create_unknown(self):
        with pytest.raises(ValueError, match="未知上下文引擎"):
            create_engine("nonexistent")



# -*- coding: utf-8 -*-
"""
Esc 键中断监听器
任务执行期间后台监听 Esc 键，主线程在检查点轮询取消标志。
"""
import sys
import threading


class EscInterrupt:
    """后台监听 Esc 键的中断信号源。

    任务开始时调用 start() 启动监听线程；
    主线程在检查点调用 is_cancelled() 轮询；
    任务结束调用 stop() 停止监听并重置状态。
    """

    def __init__(self):
        self._cancel_event = threading.Event()
        self._thread = None
        self._running = False

    def start(self):
        """启动 Esc 监听线程（仅 Windows 有效）。"""
        if sys.platform != "win32":
            return
        self._cancel_event.clear()
        self._running = True
        self._thread = threading.Thread(target=self._listen, daemon=True)
        self._thread.start()

    def stop(self):
        """停止监听并重置取消标志。"""
        self._running = False
        self._cancel_event.clear()
        # 监听线程在 getwch() 阻塞，daemon=True 会在主线程退出时自动结束

    def is_cancelled(self) -> bool:
        """主线程检查是否收到中断信号。"""
        return self._cancel_event.is_set()

    def _listen(self):
        """后台线程：轮询 Esc 键（不消耗控制台输入事件）。

        重要：不能用 msvcrt.getwch() 阻塞读，因为 stop() 后线程仍阻塞在
        getwch() 上，会和 ansi_prompt 的 getwch() 争抢控制台输入，
        导致下一轮对话的首个字符被后台线程偷走吃掉。
        改用 GetAsyncKeyState 轮询，不触碰控制台输入缓冲区。
        """
        import ctypes
        import time

        VK_ESCAPE = 0x1B
        was_pressed = False

        while self._running:
            try:
                pressed = (ctypes.windll.user32.GetAsyncKeyState(VK_ESCAPE) & 0x8000) != 0
                if pressed and not was_pressed:
                    self._cancel_event.set()
                    self._running = False
                    return
                was_pressed = pressed
                time.sleep(0.05)
            except Exception:
                break



# -*- coding: utf-8 -*-
"""memory_save / memory_search 工具单元测试"""
import os
import sys
from pathlib import Path
from unittest.mock import MagicMock

import pytest

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "..", "src"))

from abyss.memory.store import MemoryStore, MemoryEntry
from abyss.memory.bm25 import BM25Engine
from abyss.tools.memory_save import MemorySaveTool
from abyss.tools.memory_search import MemorySearchTool


@pytest.fixture
def store(tmp_path):
    return MemoryStore(tmp_path / "memory.jsonl")


class TestMemorySaveTool:
    def test_save_success(self, store):
        tool = MemorySaveTool(store=store)
        result = tool.execute(
            title="Python 调试技巧",
            content="使用 pdb 进行断点调试",
            category="fact",
            tags=["python", "debug"],
        )
        assert result["success"] is True
        assert "id" in result
        assert result["title"] == "Python 调试技巧"
        assert result["category"] == "fact"
        # 已持久化
        assert store.count == 1

    def test_save_minimal_args(self, store):
        tool = MemorySaveTool(store=store)
        result = tool.execute(title="测试", content="内容")
        assert result["success"] is True
        assert result["category"] == "session"  # 默认值

    def test_save_empty_title(self, store):
        tool = MemorySaveTool(store=store)
        result = tool.execute(title="", content="内容")
        assert result["success"] is False
        assert "title" in result["error"]

    def test_save_empty_content(self, store):
        tool = MemorySaveTool(store=store)
        result = tool.execute(title="标题", content="")
        assert result["success"] is False
        assert "content" in result["error"]

    def test_save_invalid_category_defaults_to_session(self, store):
        tool = MemorySaveTool(store=store)
        result = tool.execute(
            title="t", content="c", category="invalid_category"
        )
        assert result["success"] is True
        assert result["category"] == "session"

    def test_save_with_no_store(self):
        """store 为 None 时返回错误而非抛异常"""
        tool = MemorySaveTool(store=None)
        result = tool.execute(title="t", content="c")
        assert result["success"] is False
        assert "未加载" in result["error"]

    def test_save_tags_string_input(self, store):
        """tags 字符串应被规整"""
        tool = MemorySaveTool(store=store)
        # LLM 可能传 JSON 字符串
        result = tool.execute(
            title="t", content="c", tags='["tag1", "tag2"]'
        )
        assert result["success"] is True

    def test_tool_interface(self, store):
        """Tool 接口：name / description / parameters / to_openai_schema"""
        tool = MemorySaveTool(store=store)
        assert tool.name == "memory_save"
        assert isinstance(tool.description, str)
        assert len(tool.description) > 0
        params = tool.parameters
        assert params["type"] == "object"
        assert "title" in params["properties"]
        assert "content" in params["properties"]
        assert "title" in params["required"]
        assert "content" in params["required"]

        schema = tool.to_openai_schema()
        assert schema["type"] == "function"
        assert schema["function"]["name"] == "memory_save"


class TestMemorySearchTool:
    def test_search_basic(self, store):
        """基础检索"""
        store.add(MemoryEntry(id="m1", title="Python 入门", content="Python 编程基础"))
        store.add(MemoryEntry(id="m2", title="Java 入门", content="Java 编程基础"))
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="Python", top_k=2)
        assert result["success"] is True
        assert len(result["results"]) >= 1
        # 应返回 Python 相关
        assert any("Python" in r["title"] for r in result["results"])

    def test_search_empty_store(self, store):
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="foo")
        assert result["success"] is True
        assert result["results"] == []

    def test_search_no_match(self, store):
        store.add(MemoryEntry(id="m1", title="foo", content="bar"))
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="nonexistent")
        assert result["success"] is True
        assert result["results"] == []

    def test_search_empty_query(self, store):
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="")
        assert result["success"] is False
        assert "query" in result["error"]

    def test_search_top_k_limit(self, store):
        """top_k 限制"""
        for i in range(5):
            store.add(MemoryEntry(
                id=f"m{i}",
                title=f"Python 入门 {i}",
                content=f"Python 编程 {i}",
            ))
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="Python", top_k=2)
        assert result["success"] is True
        assert len(result["results"]) <= 2

    def test_search_top_k_max_cap(self, store):
        """top_k 超过 10 应被截断到 10"""
        for i in range(15):
            store.add(MemoryEntry(id=f"m{i}", title=f"t{i}", content=f"python code {i}"))
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="python", top_k=100)
        assert result["success"] is True
        assert len(result["results"]) <= 10

    def test_search_with_reranker(self, store):
        """启用 reranker 时调用 reranker.rerank"""
        store.add(MemoryEntry(id="m1", title="Python 入门", content="Python 基础"))
        store.add(MemoryEntry(id="m2", title="Python 进阶", content="Python 高级特性"))
        reranker = MagicMock()
        reranker.enabled = True
        # reranker 返回 [1, 0]（即倒序）
        reranker.rerank.return_value = [1, 0]
        tool = MemorySearchTool(store=store, reranker=reranker)
        result = tool.execute(query="Python", top_k=2)
        assert result["success"] is True
        assert len(result["results"]) == 2
        # 应按 reranker 返回的顺序：[1, 0] → 第二条在前
        assert result["results"][0]["id"] == "m2"
        assert result["results"][1]["id"] == "m1"
        reranker.rerank.assert_called_once()

    def test_search_reranker_disabled(self, store):
        """reranker.enabled=False 时跳过 rerank"""
        store.add(MemoryEntry(id="m1", title="Python 入门", content="Python 基础"))
        store.add(MemoryEntry(id="m2", title="Python 进阶", content="Python 高级"))
        reranker = MagicMock()
        reranker.enabled = False
        tool = MemorySearchTool(store=store, reranker=reranker)
        result = tool.execute(query="Python", top_k=2)
        assert result["success"] is True
        # reranker 不应被调用
        reranker.rerank.assert_not_called()

    def test_search_reranker_exception_fallback(self, store):
        """reranker 抛异常时应降级到 BM25 顺序"""
        store.add(MemoryEntry(id="m1", title="Python 入门", content="Python 基础"))
        store.add(MemoryEntry(id="m2", title="Python 进阶", content="Python 高级"))
        reranker = MagicMock()
        reranker.enabled = True
        reranker.rerank.side_effect = RuntimeError("LLM 失败")
        tool = MemorySearchTool(store=store, reranker=reranker)
        result = tool.execute(query="Python", top_k=2)
        assert result["success"] is True
        assert len(result["results"]) >= 1

    def test_search_with_no_store(self):
        tool = MemorySearchTool(store=None)
        result = tool.execute(query="foo")
        assert result["success"] is False
        assert "未加载" in result["error"]

    def test_search_string_top_k(self, store):
        """top_k 字符串应被规整为 int"""
        store.add(MemoryEntry(id="m1", title="python", content="code"))
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="python", top_k="3")
        assert result["success"] is True

    def test_search_cjk_query(self, store):
        """中文查询"""
        store.add(MemoryEntry(id="m1", title="调试技巧", content="使用 pdb 调试 Python"))
        store.add(MemoryEntry(id="m2", title="测试方法", content="使用 pytest 测试"))
        tool = MemorySearchTool(store=store)
        result = tool.execute(query="调试", top_k=2)
        assert result["success"] is True
        # 应优先匹配调试相关
        assert any("调试" in r["title"] for r in result["results"])

    def test_tool_interface(self, store):
        tool = MemorySearchTool(store=store)
        assert tool.name == "memory_search"
        assert isinstance(tool.description, str)
        params = tool.parameters
        assert params["type"] == "object"
        assert "query" in params["properties"]
        assert "query" in params["required"]
        schema = tool.to_openai_schema()
        assert schema["function"]["name"] == "memory_search"

    def test_custom_bm25_factory(self, store):
        """支持注入自定义 BM25Engine 工厂"""
        mock_engine = MagicMock()
        mock_engine.search.return_value = []
        mock_engine.index.return_value = None
        tool = MemorySearchTool(
            store=store,
            bm25_factory=lambda: mock_engine,
        )
        store.add(MemoryEntry(id="m1", title="t", content="c"))
        result = tool.execute(query="foo")
        assert result["success"] is True
        mock_engine.index.assert_called_once()
        mock_engine.search.assert_called_once()



# -*- coding: utf-8 -*-
"""CM AiAnalysisCommands AI 分析命令族（Phase 8 Tier 2）

借鉴 zhikuncode AiAnalysisCommands.java（70 行）：
- /bug-hunter (别名 /bughunt) — 智能扫描项目潜在 Bug
- /pr-review (别名 /review-pr) — 审查 Git diff 或指定 PR
- /security-scan (别名 /sec-scan) — 扫描安全漏洞
- /explain — 解释代码片段或概念

实现模式：复用 _inject_methodology 模式，每条命令生成结构化 prompt 注入 session。
集成点：main.py handle_slash_command 注册 4 条命令 + 3 个别名。
"""

# Bug Hunter Prompt：扫描项目潜在 Bug
BUG_HUNTER_PROMPT = """# Bug Hunter 模式

你现在是 Bug Hunter 模式。请扫描项目代码，识别以下类型的潜在 Bug：

## 扫描维度

1. **空指针 / None 引用**：未检查的 None 访问、未初始化变量
2. **资源泄漏**：未关闭的文件/连接/锁、未释放的内存
3. **并发问题**：竞态条件、死锁、可见性问题、未同步的共享状态
4. **边界条件**：数组越界、整数溢出、空集合操作
5. **异常处理**：吞异常、过宽的 except、未传播的关键错误
6. **逻辑错误**：错误的条件判断、死代码、不可达分支
7. **类型错误**：运行时类型不匹配、隐式类型转换陷阱
8. **资源管理**：忘记释放、双重释放、错误的清理顺序

## 输出格式

对每个发现的 Bug：
- **位置**：文件路径 + 行号
- **类型**：上述 8 类之一
- **严重性**：Critical / High / Medium / Low
- **描述**：问题简述
- **建议**：修复方案

## 严格规则

- 仅报告真实存在的 Bug，不臆测
- 优先级 Critical > High > Medium > Low
- 同一问题只报告一次（取最严重等级）
- 对每个建议给出最小改动方案

---
"""

# PR Review Prompt：审查 Git diff 或指定 PR
PR_REVIEW_PROMPT = """# PR Review 模式

你现在是 PR Review 模式。请按以下维度审查代码变更：

## 评审维度

1. **正确性**：逻辑是否正确？是否处理所有边界？
2. **可读性**：命名是否清晰？结构是否直观？是否需要注释？
3. **代码规范**：是否符合项目既有风格？
4. **安全性**：是否引入安全漏洞（注入/越权/敏感信息泄漏）？
5. **性能**：是否有不必要的开销？算法复杂度合理？
6. **测试覆盖**：是否需要新增/修改测试？
7. **可维护性**：是否过度抽象？是否符合 YAGNI？
8. **依赖**：是否引入不必要依赖？是否破坏向后兼容？

## 输出格式

对每个变更文件：
- **文件**：路径
- **评级**：Approve / Request Changes / Block
- **问题**：按维度列出（如有）
- **建议**：具体修改建议（含代码示例）

## 严格规则

- 区分"必须修改"和"建议修改"
- 不强推个人偏好（如命名风格遵循项目既有）
- 对好评也要明确指出（不只是挑刺）

---
"""

# Security Scan Prompt：扫描安全漏洞（OWASP Top 10）
SECURITY_SCAN_PROMPT = """# Security Scan 模式

你现在是 Security Scan 模式。请按 OWASP Top 10 类别扫描项目安全漏洞：

## OWASP Top 10 扫描类别

1. **A01 - Broken Access Control**：越权访问、垂直/水平权限漏洞
2. **A02 - Cryptographic Failures**：弱加密、硬编码密钥、不安全的随机数
3. **A03 - Injection**：SQL 注入、Command 注入、LDAP 注入、XPath 注入
4. **A04 - Insecure Design**：缺失速率限制、未保护的关键路径
5. **A05 - Security Misconfiguration**：默认配置、详细错误信息、不必要的特性开启
6. **A06 - Vulnerable and Outdated Components**：依赖中的已知 CVE
7. **A07 - Identification and Authentication Failures**：弱密码、会话固定、缺失 MFA
8. **A08 - Software and Data Integrity Failures**：未签名更新、反序列化漏洞
9. **A09 - Security Logging and Monitoring Failures**：缺失审计日志、未监控关键事件
10. **A10 - Server-Side Request Forgery (SSRF)**：未校验的 URL 请求、内网访问

## 额外检查

- 硬编码密码 / API Key / Token
- 路径穿越（../）
- XSS（反射型/存储型/DOM 型）
- CSRF Token 缺失
- 不安全的反序列化
- 敏感信息日志泄漏

## 输出格式

对每个发现的漏洞：
- **位置**：文件路径 + 行号
- **OWASP 类别**：上述 10 类之一
- **严重性**：Critical / High / Medium / Low
- **PoC**：可复现的攻击向量（如适用）
- **修复建议**：具体代码修改方案

## 严格规则

- 仅报告真实漏洞，不臆测
- 优先报告 Critical 和 High
- 对每个建议给出最小改动方案
- 区分"已验证"和"疑似"

---
"""

# Explain Prompt：解释代码或概念
EXPLAIN_PROMPT = """# Explain 模式

你现在是 Explain 模式。请按以下要求解释用户指定的代码或概念：

## 解释维度

1. **是什么**：一句话总结目标代码/概念的作用
2. **怎么做**：执行流程、关键步骤、数据流向
3. **为什么**：设计动机、对比替代方案的优劣
4. **关键点**：易错点、边界条件、性能考量
5. **示例**：最小可运行示例（如适用）
6. **延伸**：相关概念、官方文档链接、进一步学习方向

## 严格规则

- 用通俗易懂的语言，避免堆砌术语
- 必要时画 ASCII 流程图或时序图
- 标注关键代码行（用 `// ← 关键` 注释）
- 不臆测，基于代码实际行为解释

---
"""

# 命令注册表：cmd -> prompt
AI_ANALYSIS_COMMANDS = {
    "/bug-hunter": BUG_HUNTER_PROMPT,
    "/pr-review": PR_REVIEW_PROMPT,
    "/security-scan": SECURITY_SCAN_PROMPT,
    "/explain": EXPLAIN_PROMPT,
}

# 别名表：alias -> canonical
ALIASES = {
    "/bughunt": "/bug-hunter",
    "/review-pr": "/pr-review",
    "/sec-scan": "/security-scan",
}


def handle_ai_analysis(cmd: str, user_desc: str, session, working_dir: str) -> str:
    """AI 分析命令处理：注入结构化 prompt 到 session。

    Args:
        cmd: 命令名（已解析别名为 canonical）
        user_desc: 用户附加描述
        session: Session 实例，需有 add_user 方法
        working_dir: 工作目录路径

    Returns:
        "injected" — 已注入，主循环将其作为新 prompt
        错误消息字符串 — 未注入，直接显示给用户
    """
    # 解析别名
    canonical = ALIASES.get(cmd, cmd)
    prompt = AI_ANALYSIS_COMMANDS.get(canonical)
    if prompt is None:
        return f"[ERR] 未知命令: {cmd}（可用: {', '.join(AI_ANALYSIS_COMMANDS.keys())}）"

    # /explain 必须带目标
    if canonical == "/explain" and not user_desc.strip():
        return "用法: /explain <代码片段或概念>"

    # 组合 prompt + 用户描述
    combined = f"{prompt}\n---\n\n# 用户任务\n{user_desc}" if user_desc.strip() else prompt
    session.add_user(combined)
    return "injected"



# -*- coding: utf-8 -*-
"""
回归测试：tool_calls 流式生成期间必须显示 spinner，避免终端看似卡死。
"""
import io
import re
import sys
import time
from unittest.mock import patch, MagicMock
import pytest

import abyss.main as main_mod


def _render_terminal(raw_text):
    """模拟终端渲染，返回可见行列表（处理 \\r \\n \\x1b[K ANSI SGR）。"""
    text = re.sub(r'\x1b\[[0-9;]*[a-zA-Z]', '', raw_text)
    lines = [""]
    cur = 0
    pos = 0
    i = 0
    while i < len(text):
        ch = text[i]
        if ch == '\r':
            pos = 0
            i += 1
        elif ch == '\n':
            lines.append("")
            cur += 1
            pos = 0
            i += 1
        else:
            line = lines[cur]
            if pos < len(line):
                lines[cur] = line[:pos] + ch + line[pos+1:]
            else:
                lines[cur] = line + ' ' * (pos - len(line)) + ch
            pos += 1
            i += 1
    return lines


def _make_tc_delta(index=0, name=None, args=""):
    """创建一个 tool_calls delta chunk 片段。"""
    return {"index": index, "id": f"call_{index}", "function": {"name": name, "arguments": args}}


def _run_consume_stream(chunks, show_reasoning=True):
    """运行 _consume_stream 并捕获 stdout，返回 (raw_output, visible_lines, result)。"""
    buf = io.StringIO()
    old_stdout = sys.stdout
    sys.stdout = buf
    spinner = main_mod.Spinner("思考中")
    try:
        with patch.object(main_mod, '_ESC', None), \
             patch.object(main_mod.logger, 'api_first_token', MagicMock()), \
             patch.object(main_mod.logger, 'api_response', MagicMock()), \
             patch.object(main_mod.logger, 'stream_reasoning_start', MagicMock()), \
             patch.object(main_mod.logger, 'stream_content_start', MagicMock()), \
             patch.object(main_mod.logger, 'spinner_start', MagicMock()), \
             patch.object(main_mod.logger, 'spinner_stop', MagicMock()), \
             patch('time.sleep', return_value=None):
            spinner.start()
            result = main_mod._consume_stream(iter(chunks), spinner, show_reasoning)
    finally:
        spinner.stop()
        sys.stdout = old_stdout

    raw = buf.getvalue()
    return raw, _render_terminal(raw), result


class TestToolCallsSpinner:
    """验证 tool_calls 流式生成期间显示「生成中」spinner。"""

    def test_spinner_shows_during_tool_calls_after_content(self):
        """先输出 content，再输出 tool_calls → 生成中 spinner 必须出现。"""
        chunks = [
            {"reasoning_content": None, "content": "我来帮你修改文件。", "tool_calls": None},
            {"reasoning_content": None, "content": None, "tool_calls": [_make_tc_delta(name="file_write")]},
            {"reasoning_content": None, "content": None, "tool_calls": [{"index": 0, "function": {"arguments": "{}"}}]},
        ]
        raw, lines, result = _run_consume_stream(chunks)
        content, reasoning, tool_calls, _usage = result

        assert len(tool_calls) > 0, "应该收集到 tool_calls"
        assert "生成中" in raw, (
            f"tool_calls 生成期间应显示「生成中」spinner！\n原始输出: {raw!r}"
        )

    def test_spinner_shows_during_tool_calls_after_reasoning(self):
        """先输出 reasoning，再输出 tool_calls（无 content）→ 生成中 spinner 必须出现。"""
        chunks = [
            {"reasoning_content": "用户要求修改，我需要先读取文件。", "content": None, "tool_calls": None},
            {"reasoning_content": None, "content": None, "tool_calls": [_make_tc_delta(name="file_write")]},
        ]
        raw, lines, result = _run_consume_stream(chunks)
        content, reasoning, tool_calls, _usage = result

        assert len(reasoning) > 0, "应该有 reasoning"
        assert len(tool_calls) > 0, "应该收集到 tool_calls"
        assert "生成中" in raw, (
            f"reasoning 后直接 tool_calls 时应显示「生成中」spinner！\n原始输出: {raw!r}"
        )

    def test_spinner_shows_when_tool_calls_first(self):
        """无 reasoning 无 content，直接 tool_calls → 生成中 spinner 必须出现。"""
        chunks = [
            {"reasoning_content": None, "content": None, "tool_calls": [_make_tc_delta(name="shell_exec")]},
        ]
        raw, lines, result = _run_consume_stream(chunks, show_reasoning=False)
        content, reasoning, tool_calls, _usage = result

        assert len(tool_calls) > 0, "应该收集到 tool_calls"
        assert "生成中" in raw, (
            f"首个输出为 tool_calls 时应显示「生成中」spinner！\n原始输出: {raw!r}"
        )

    def test_no_extra_spinner_for_pure_content(self):
        """纯 content 回复（无 tool_calls）→ 不应出现「生成中」spinner。"""
        chunks = [
            {"reasoning_content": None, "content": "这是纯文本回复，没有工具调用。", "tool_calls": None},
        ]
        raw, lines, result = _run_consume_stream(chunks)
        content, reasoning, tool_calls, _usage = result

        assert len(content) > 0, "应该有 content"
        assert len(tool_calls) == 0, "不应有 tool_calls"
        assert "生成中" not in raw, (
            f"纯 content 回复不应出现「生成中」spinner！\n原始输出: {raw!r}"
        )

    def test_collects_all_tool_call_delta_chunks(self):
        """多个 tool_calls delta chunk 应被正确收集。"""
        chunks = [
            {"reasoning_content": None, "content": "开始写文件。", "tool_calls": None},
            {"reasoning_content": None, "content": None, "tool_calls": [_make_tc_delta(name="file_write")]},
            {"reasoning_content": None, "content": None, "tool_calls": [{"index": 0, "function": {"arguments": "{}"}}]},
        ]
        raw, lines, result = _run_consume_stream(chunks)
        content, reasoning, tool_calls, _usage = result

        assert len(tool_calls) == 2, f"应收集2个 tool_calls delta，实际 {len(tool_calls)}"
        assert "生成中" in raw, "生成中 spinner 应出现"
